
import numpy as np

import ase.gui.ui as ui
from ase.gui.i18n import _


class Movie:
    def __init__(self, gui):
        self.win = win = ui.Window(_('Movie'), self.close)
        win.add(_('Image number:'))
        self.frame_number = ui.Scale(gui.frame + 1, 1,
                                     len(gui.images),
                                     callback=self.new_frame)
        win.add(self.frame_number)

        win.add([ui.Button(_('First'), self.click, -1, True),
                 ui.Button(_('Back'), self.click, -1),
                 ui.Button(_('Forward'), self.click, 1),
                 ui.Button(_('Last'), self.click, 1, True)])

        play = ui.Button(_('Play'), self.play)
        stop = ui.Button(_('Stop'), self.stop)

        # TRANSLATORS: This function plays an animation forwards and backwards
        # alternatingly, e.g. for displaying vibrational movement
        self.rock = ui.CheckButton(_('Rock'))

        win.add([play, stop, self.rock])

        if len(gui.images) > 150:
            skipdefault = len(gui.images) // 150
            tdefault = min(max(len(gui.images) / (skipdefault * 5.0),
                               1.0), 30)
        else:
            skipdefault = 0
            tdefault = min(max(len(gui.images) / 5.0, 1.0), 30)
        self.time = ui.SpinBox(tdefault, 1.0, 99, 0.1)
        self.skip = ui.SpinBox(skipdefault, 0, 99, 1)
        win.add([_(' Frame rate: '), self.time, _(' Skip frames: '),
                 self.skip])

        self.gui = gui
        self.direction = 1
        self.timer = None
        gui.register_vulnerable(self)

    def notify_atoms_changed(self):
        """Called by gui object when the atoms have changed."""
        self.close()

    def close(self):
        self.stop()
        self.win.close()

    def click(self, step, firstlast=False):
        if firstlast and step < 0:
            i = 0
        elif firstlast:
            i = len(self.gui.images) - 1
        else:
            i = max(0, min(len(self.gui.images) - 1, self.gui.frame + step))

        self.frame_number.value = i + 1
        if firstlast:
            self.direction = np.sign(-step)
        else:
            self.direction = np.sign(step)

    def new_frame(self, value):
        self.gui.set_frame(value - 1)

    def play(self):
        self.stop()
        t = 1 / self.time.value
        self.timer = self.gui.window.after(t, self.step)

    def stop(self):
        if self.timer is not None:
            self.timer.cancel()

    def step(self):
        i = self.gui.frame
        nimages = len(self.gui.images)
        delta = int(self.skip.value + 1)

        if self.rock.value:
            if i <= self.skip.value:
                self.direction = 1
            elif i >= nimages - delta:
                self.direction = -1
            i += self.direction * delta
        else:
            i = (i + self.direction * delta + nimages) % nimages

        self.frame_number.value = i + 1
        self.play()



"""

Check the unit cell is handled correctly

"""

from ase.calculators.vasp import Vasp
from ase.build import molecule
from ase.test import must_raise

# Molecules come with no unit cell

atoms = molecule('CH4')
calc = Vasp()

with must_raise(RuntimeError):
    atoms.write('POSCAR')

with must_raise(ValueError):
    atoms.set_calculator(calc)
    atoms.get_total_energy()



"""Verify 2D Bravais lattice and band path versus pbc information."""

from ase.cell import Cell

cell = Cell([[1.,0.,0.],
             [.1,1.,0.],
             [0.,0.,0.]])
lat = cell.get_bravais_lattice()
print(cell.cellpar())
print(lat)
assert lat.name == 'OBL'

cell[2, 2] = 7
lat3d = cell.get_bravais_lattice()
print(lat3d)
assert lat3d.name == 'MCL'
lat2d_pbc = cell.get_bravais_lattice(pbc=[1, 1, 0])
print(lat2d_pbc)
assert lat2d_pbc.name == 'OBL'

path = cell.bandpath()
print(path)

path2d = cell.bandpath(pbc=[1, 1, 0])
print(path2d)
assert path2d.cell.rank == 2
assert path2d.cell.get_bravais_lattice().name == 'OBL'



#test the geometry.analysis module
import numpy as np
from ase.geometry.analysis import Analysis
from ase.build import molecule

mol = molecule('CH3CH2OH')
ana = Analysis(mol)
assert np.shape(ana.adjacency_matrix[0].todense()) == (9, 9)
for imI in range(len(ana.all_bonds)):
    l1 = sum([len(x) for x in ana.all_bonds[imI]])
    l2 = sum([len(x) for x in ana.unique_bonds[imI]])
    assert l1 == l2 * 2

for imi in range(len(ana.all_angles)):
    l1 = sum([len(x) for x in ana.all_angles[imi]])
    l2 = sum([len(x) for x in ana.unique_angles[imi]])
    assert l1 == l2 * 2

for imi in range(len(ana.all_dihedrals)):
    l1 = sum([len(x) for x in ana.all_dihedrals[imi]])
    l2 = sum([len(x) for x in ana.unique_dihedrals[imi]])
    assert l1 == l2 * 2

assert len(ana.get_angles('C','C','H', unique=False)[0]) == len(ana.get_angles('C','C','H', unique=True)[0])*2


csixty = molecule('C60')
mol = molecule('C7NH5')

ana = Analysis(csixty)
ana2 = Analysis(mol)
for imI in range(len(ana.all_bonds)):
    l1 = sum([len(x) for x in ana.all_bonds[imI]])
    l2 = sum([len(x) for x in ana.unique_bonds[imI]])
    assert l1 == l2 * 2
for imI in range(len(ana.all_angles)):
    l1 = sum([len(x) for x in ana.all_angles[imI]])
    l2 = sum([len(x) for x in ana.unique_angles[imI]])
    assert l1 == l2 * 2
for imI in range(len(ana.all_dihedrals)):
    l1 = sum([len(x) for x in ana.all_dihedrals[imI]])
    l2 = sum([len(x) for x in ana.unique_dihedrals[imI]])
    assert l1 == l2 * 2

assert len(ana2.get_angles('C','C','H', unique=False)[0]) == len(ana2.get_angles('C','C','H', unique=True)[0]) * 2
assert len(ana2.get_dihedrals('H','C','C','H',unique=False)[0]) == len(ana2.get_dihedrals('H','C','C','H',unique=True)[0]) * 2





'''
Conversion of text from a Crystallographic Information File (CIF) format to
unicode. CIF text is neither unicode nor bibtex/latex code.

Rules for character formatting in CIF files are specified at:
https://www.iucr.org/resources/cif/spec/version1.1/semantics
'''

import re
import html


subs_dict = {
    '\r': '',            # Windows line ending
    '\t': ' ',           # tabs

    r'\a': u'\u03b1',    # alpha
    r'\b': u'\u03b2',    # beta
    r'\g': u'\u03b3',    # gamma
    r'\d': u'\u03b4',    # delta
    r'\e': u'\u03b5',    # epsilon
    r'\z': u'\u03b6',    # zeta
    r'\h': u'\u03b7',    # eta
    r'\q': u'\u03b8',    # theta
    r'\i': u'\u03b9',    # iota
    r'\k': u'\u03ba',    # kappa
    r'\l': u'\u03bb',    # lambda
    r'\m': u'\u03bc',    # mu
    r'\n': u'\u03bd',    # nu
    r'\x': u'\u03be',    # xi
    r'\o': u'\u03bf',    # omicron
    r'\p': u'\u03c0',    # pi
    r'\r': u'\u03c1',    # rho
    r'\s': u'\u03c3',    # sigma
    r'\t': u'\u03c4',    # tau
    r'\u': u'\u03c5',    # upsilon
    r'\f': u'\u03c6',    # phi
    r'\c': u'\u03c7',    # chi
    r'\y': u'\u03c8',    # psi
    r'\w': u'\u03c9',    # omega
    r'\A': u'\u0391',    # Alpha
    r'\B': u'\u0392',    # Beta
    r'\G': u'\u0393',    # Gamma
    r'\D': u'\u0394',    # Delta
    r'\E': u'\u0395',    # Epsilon
    r'\Z': u'\u0396',    # Zeta
    r'\H': u'\u0397',    # Eta
    r'\Q': u'\u0398',    # Theta
    r'\I': u'\u0399',    # Ioto
    r'\K': u'\u039a',    # Kappa
    r'\L': u'\u039b',    # Lambda
    r'\M': u'\u039c',    # Mu
    r'\N': u'\u039d',    # Nu
    r'\X': u'\u039e',    # Xi
    r'\O': u'\u039f',    # Omicron
    r'\P': u'\u03a0',    # Pi
    r'\R': u'\u03a1',    # Rho
    r'\S': u'\u03a3',    # Sigma
    r'\T': u'\u03a4',    # Tau
    r'\U': u'\u03a5',    # Upsilon
    r'\F': u'\u03a6',    # Phi
    r'\C': u'\u03a7',    # Chi
    r'\Y': u'\u03a8',    # Psi
    r'\W': u'\u03a9',    # Omega

    r'\%a': u'\u00e5',   # a-ring
    r'\/o': u'\u00f8',   # o-slash
    r'\?i': u'\u0131',   # dotless i
    r'\/l': u'\u0142',   # Polish l
    r'\&s': u'\u00df',   # German eszett
    r'\/d': u'\u0111',   # barred d

    r'\%A': u'\u00c5',   # A-ring
    r'\/O': u'\u00d8',   # O-slash
    r'\?I': 'I',         # dotless I
    r'\/L': u'\u0141',   # Polish L
    r'\&S': u'\u1e9e',   # German Eszett
    r'\/D': u'\u0110',   # barred D

    r'\%': u'\u00b0',           # degree
    r'--': u'\u2013',           # dash
    r'---': u'\u2014',          # single bond
    r'\\db': u'\u003d',         # double bond
    r'\\tb': u'\u2261',         # triple bond
    r'\\ddb': u'\u2248',        # delocalized double bond
    r'\\sim': '~',
    r'\\simeq': u'\u2243',
    r'\\infty': u'\u221e',      # infinity

    r'\\times': u'\u00d7',
    r'+-': u'\u00b1',           # plusminus
    r'-+': u'\u2213',           # minusplus
    r'\\square': u'\u25a0',
    r'\\neq': u'\u2660',
    r'\\rangle': u'\u3009',
    r'\\langle': u'\u3008',
    r'\\rightarrow': u'\u2192',
    r'\\leftarrow': u'\u2190',

    r"\'A": u'\u00c1',  # A acute
    r"\'E": u'\u00c9',  # E acute
    r"\'I": u'\u00cd',  # I acute
    r"\'O": u'\u00d3',  # O acute
    r"\'U": u'\u00da',  # U acute
    r"\'Y": u'\u00dd',  # Y acute
    r"\'a": u'\u00e1',  # a acute
    r"\'e": u'\u00e9',  # e acute
    r"\'i": u'\u00ed',  # i acute
    r"\'o": u'\u00f3',  # o acute
    r"\'u": u'\u00fa',  # u acute
    r"\'y": u'\u00fd',  # y acute
    r"\'C": u'\u0106',  # C acute
    r"\'c": u'\u0107',  # c acute
    r"\'L": u'\u0139',  # L acute
    r"\'l": u'\u013a',  # l acute
    r"\'N": u'\u0143',  # N acute
    r"\'n": u'\u0144',  # n acute
    r"\'R": u'\u0154',  # R acute
    r"\'r": u'\u0155',  # r acute
    r"\'S": u'\u015a',  # S acute
    r"\'s": u'\u015b',  # s acute
    r"\'Z": u'\u0179',  # Z acute
    r"\'z": u'\u017a',  # z acute
    r"\'G": u'\u01f4',  # G acute
    r"\'g": u'\u01f5',  # g acute
    r"\'K": u'\u1e30',  # K acute
    r"\'k": u'\u1e31',  # k acute
    r"\'M": u'\u1e3e',  # M acute
    r"\'m": u'\u1e3f',  # m acute
    r"\'P": u'\u1e54',  # P acute
    r"\'p": u'\u1e55',  # p acute
    r"\'W": u'\u1e82',  # W acute
    r"\'w": u'\u1e83',  # w acute
    r'\;A': u'\u0104',  # A ogonek
    r'\;a': u'\u0105',  # a ogonek
    r'\;E': u'\u0118',  # E ogonek
    r'\;e': u'\u0119',  # e ogonek
    r'\;I': u'\u012e',  # I ogonek
    r'\;i': u'\u012f',  # i ogonek
    r'\;U': u'\u0172',  # U ogonek
    r'\;u': u'\u0173',  # u ogonek
    r'\;O': u'\u01ea',  # O ogonek
    r'\;o': u'\u01eb',  # o ogonek
    r'\.C': u'\u010a',  # C dot above
    r'\.c': u'\u010b',  # c dot above
    r'\.E': u'\u0116',  # E dot above
    r'\.e': u'\u0117',  # e dot above
    r'\.G': u'\u0120',  # G dot above
    r'\.g': u'\u0121',  # g dot above
    r'\.I': u'\u0130',  # I dot above
    r'\.Z': u'\u017b',  # Z dot above
    r'\.z': u'\u017c',  # z dot above
    r'\.A': u'\u0226',  # A dot above
    r'\.a': u'\u0227',  # a dot above
    r'\.O': u'\u022e',  # O dot above
    r'\.o': u'\u022f',  # o dot above
    r'\.B': u'\u1e02',  # B dot above
    r'\.b': u'\u1e03',  # b dot above
    r'\.D': u'\u1e0a',  # D dot above
    r'\.d': u'\u1e0b',  # d dot above
    r'\.F': u'\u1e1e',  # F dot above
    r'\.f': u'\u1e1f',  # f dot above
    r'\.H': u'\u1e22',  # H dot above
    r'\.h': u'\u1e23',  # h dot above
    r'\.M': u'\u1e40',  # M dot above
    r'\.m': u'\u1e41',  # m dot above
    r'\.N': u'\u1e44',  # N dot above
    r'\.n': u'\u1e45',  # n dot above
    r'\.P': u'\u1e56',  # P dot above
    r'\.p': u'\u1e57',  # p dot above
    r'\.R': u'\u1e58',  # R dot above
    r'\.r': u'\u1e59',  # r dot above
    r'\.S': u'\u1e60',  # S dot above
    r'\.s': u'\u1e61',  # s dot above
    r'\.T': u'\u1e6a',  # T dot above
    r'\.t': u'\u1e6b',  # t dot above
    r'\.W': u'\u1e86',  # W dot above
    r'\.w': u'\u1e87',  # w dot above
    r'\.X': u'\u1e8a',  # X dot above
    r'\.x': u'\u1e8b',  # x dot above
    r'\.Y': u'\u1e8e',  # Y dot above
    r'\.y': u'\u1e8f',  # y dot above
    r'\(A': u'\u0102',  # A breve
    r'\(a': u'\u0103',  # a breve
    r'\(E': u'\u0114',  # E breve
    r'\(e': u'\u0115',  # e breve
    r'\(G': u'\u011e',  # G breve
    r'\(g': u'\u011f',  # g breve
    r'\(I': u'\u012c',  # I breve
    r'\(i': u'\u012d',  # i breve
    r'\(O': u'\u014e',  # O breve
    r'\(o': u'\u014f',  # o breve
    r'\(U': u'\u016c',  # U breve
    r'\(u': u'\u016d',  # u breve
    r'\=A': u'\u0100',  # A macron
    r'\=a': u'\u0101',  # a macron
    r'\=E': u'\u0112',  # E macron
    r'\=e': u'\u0113',  # e macron
    r'\=I': u'\u012a',  # I macron
    r'\=i': u'\u012b',  # i macron
    r'\=O': u'\u014c',  # O macron
    r'\=o': u'\u014d',  # o macron
    r'\=U': u'\u016a',  # U macron
    r'\=u': u'\u016b',  # u macron
    r'\=Y': u'\u0232',  # Y macron
    r'\=y': u'\u0233',  # y macron
    r'\=G': u'\u1e20',  # G macron
    r'\=g': u'\u1e21',  # g macron
    r'\^A': u'\u00c2',  # A circumflex
    r'\^E': u'\u00ca',  # E circumflex
    r'\^I': u'\u00ce',  # I circumflex
    r'\^O': u'\u00d4',  # O circumflex
    r'\^U': u'\u00db',  # U circumflex
    r'\^a': u'\u00e2',  # a circumflex
    r'\^e': u'\u00ea',  # e circumflex
    r'\^i': u'\u00ee',  # i circumflex
    r'\^o': u'\u00f4',  # o circumflex
    r'\^u': u'\u00fb',  # u circumflex
    r'\^C': u'\u0108',  # C circumflex
    r'\^c': u'\u0109',  # c circumflex
    r'\^G': u'\u011c',  # G circumflex
    r'\^g': u'\u011d',  # g circumflex
    r'\^H': u'\u0124',  # H circumflex
    r'\^h': u'\u0125',  # h circumflex
    r'\^J': u'\u0134',  # J circumflex
    r'\^j': u'\u0135',  # j circumflex
    r'\^S': u'\u015c',  # S circumflex
    r'\^s': u'\u015d',  # s circumflex
    r'\^W': u'\u0174',  # W circumflex
    r'\^w': u'\u0175',  # w circumflex
    r'\^Y': u'\u0176',  # Y circumflex
    r'\^y': u'\u0177',  # y circumflex
    r'\^Z': u'\u1e90',  # Z circumflex
    r'\^z': u'\u1e91',  # z circumflex
    r'\"A': u'\u00c4',  # A diaeresis
    r'\"E': u'\u00cb',  # E diaeresis
    r'\"I': u'\u00cf',  # I diaeresis
    r'\"O': u'\u00d6',  # O diaeresis
    r'\"U': u'\u00dc',  # U diaeresis
    r'\"a': u'\u00e4',  # a diaeresis
    r'\"e': u'\u00eb',  # e diaeresis
    r'\"i': u'\u00ef',  # i diaeresis
    r'\"o': u'\u00f6',  # o diaeresis
    r'\"u': u'\u00fc',  # u diaeresis
    r'\"y': u'\u00ff',  # y diaeresis
    r'\"Y': u'\u0178',  # Y diaeresis
    r'\"H': u'\u1e26',  # H diaeresis
    r'\"h': u'\u1e27',  # h diaeresis
    r'\"W': u'\u1e84',  # W diaeresis
    r'\"w': u'\u1e85',  # w diaeresis
    r'\"X': u'\u1e8c',  # X diaeresis
    r'\"x': u'\u1e8d',  # x diaeresis
    r'\"t': u'\u1e97',  # t diaeresis
    r'\~A': u'\u00c3',  # A tilde
    r'\~N': u'\u00d1',  # N tilde
    r'\~O': u'\u00d5',  # O tilde
    r'\~a': u'\u00e3',  # a tilde
    r'\~n': u'\u00f1',  # n tilde
    r'\~o': u'\u00f5',  # o tilde
    r'\~I': u'\u0128',  # I tilde
    r'\~i': u'\u0129',  # i tilde
    r'\~U': u'\u0168',  # U tilde
    r'\~u': u'\u0169',  # u tilde
    r'\~V': u'\u1e7c',  # V tilde
    r'\~v': u'\u1e7d',  # v tilde
    r'\~E': u'\u1ebc',  # E tilde
    r'\~e': u'\u1ebd',  # e tilde
    r'\~Y': u'\u1ef8',  # Y tilde
    r'\~y': u'\u1ef9',  # y tilde
    r'\<C': u'\u010c',  # C caron
    r'\<c': u'\u010d',  # c caron
    r'\<D': u'\u010e',  # D caron
    r'\<d': u'\u010f',  # d caron
    r'\<E': u'\u011a',  # E caron
    r'\<e': u'\u011b',  # e caron
    r'\<L': u'\u013d',  # L caron
    r'\<l': u'\u013e',  # l caron
    r'\<N': u'\u0147',  # N caron
    r'\<n': u'\u0148',  # n caron
    r'\<R': u'\u0158',  # R caron
    r'\<r': u'\u0159',  # r caron
    r'\<S': u'\u0160',  # S caron
    r'\<s': u'\u0161',  # s caron
    r'\<T': u'\u0164',  # T caron
    r'\<t': u'\u0165',  # t caron
    r'\<Z': u'\u017d',  # Z caron
    r'\<z': u'\u017e',  # z caron
    r'\<A': u'\u01cd',  # A caron
    r'\<a': u'\u01ce',  # a caron
    r'\<I': u'\u01cf',  # I caron
    r'\<i': u'\u01d0',  # i caron
    r'\<O': u'\u01d1',  # O caron
    r'\<o': u'\u01d2',  # o caron
    r'\<U': u'\u01d3',  # U caron
    r'\<u': u'\u01d4',  # u caron
    r'\<G': u'\u01e6',  # G caron
    r'\<g': u'\u01e7',  # g caron
    r'\<K': u'\u01e8',  # K caron
    r'\<k': u'\u01e9',  # k caron
    r'\<j': u'\u01f0',  # j caron
    r'\<H': u'\u021e',  # H caron
    r'\<h': u'\u021f',  # h caron
    r'\>O': u'\u0150',  # O double acute
    r'\>o': u'\u0151',  # o double acute
    r'\>U': u'\u0170',  # U double acute
    r'\>u': u'\u0171',  # u double acute
    r'\,C': u'\u00c7',  # C cedilla
    r'\,c': u'\u00e7',  # c cedilla
    r'\,G': u'\u0122',  # G cedilla
    r'\,g': u'\u0123',  # g cedilla
    r'\,K': u'\u0136',  # K cedilla
    r'\,k': u'\u0137',  # k cedilla
    r'\,L': u'\u013b',  # L cedilla
    r'\,l': u'\u013c',  # l cedilla
    r'\,N': u'\u0145',  # N cedilla
    r'\,n': u'\u0146',  # n cedilla
    r'\,R': u'\u0156',  # R cedilla
    r'\,r': u'\u0157',  # r cedilla
    r'\,S': u'\u015e',  # S cedilla
    r'\,s': u'\u015f',  # s cedilla
    r'\,T': u'\u0162',  # T cedilla
    r'\,t': u'\u0163',  # t cedilla
    r'\,E': u'\u0228',  # E cedilla
    r'\,e': u'\u0229',  # e cedilla
    r'\,D': u'\u1e10',  # D cedilla
    r'\,d': u'\u1e11',  # d cedilla
    r'\,H': u'\u1e28',  # H cedilla
    r'\,h': u'\u1e29',  # h cedilla
    r'\`A': u'\u00c0',  # A grave
    r'\`E': u'\u00c8',  # E grave
    r'\`I': u'\u00cc',  # I grave
    r'\`O': u'\u00d2',  # O grave
    r'\`U': u'\u00d9',  # U grave
    r'\`a': u'\u00e0',  # a grave
    r'\`e': u'\u00e8',  # e grave
    r'\`i': u'\u00ec',  # i grave
    r'\`o': u'\u00f2',  # o grave
    r'\`u': u'\u00f9',  # u grave
    r'\`N': u'\u01f8',  # N grave
    r'\`n': u'\u01f9',  # n grave
    r'\`W': u'\u1e80',  # W grave
    r'\`w': u'\u1e81',  # w grave
    r'\`Y': u'\u1ef2',  # Y grave
    r'\`y': u'\u1ef3',  # y grave
}

superscript_dict = {
    '0': u'\u2070',  # superscript 0
    '1': u'\u00b9',  # superscript 1
    '2': u'\u00b2',  # superscript 2
    '3': u'\u00b3',  # superscript 3
    '4': u'\u2074',  # superscript 4
    '5': u'\u2075',  # superscript 5
    '6': u'\u2076',  # superscript 6
    '7': u'\u2077',  # superscript 7
    '8': u'\u2078',  # superscript 8
    '9': u'\u2079',  # superscript 9
}

subscript_dict = {
    '0': u'\u2080',  # subscript 0
    '1': u'\u2081',  # subscript 1
    '2': u'\u2082',  # subscript 2
    '3': u'\u2083',  # subscript 3
    '4': u'\u2084',  # subscript 4
    '5': u'\u2085',  # subscript 5
    '6': u'\u2086',  # subscript 6
    '7': u'\u2087',  # subscript 7
    '8': u'\u2088',  # subscript 8
    '9': u'\u2089',  # subscript 9
}


def replace_subscript(s, subscript=True):

    target = '~'
    rdict = subscript_dict
    if not subscript:
        target = '^'
        rdict = superscript_dict

    replaced = []
    inside = False
    for char in s:
        if char == target:
            inside = not inside
        elif not inside:
            replaced += [char]
        # note: do not use char.isdigit - this also matches (sub/super)scripts
        elif char in rdict:
            replaced += [rdict[char]]
        else:
            replaced += [char]

    return ''.join(replaced)


def multiple_replace(text, adict):
    rx = re.compile('|'.join(map(re.escape, adict)))

    def one_xlat(match):
        return adict[match.group(0)]

    return rx.sub(one_xlat, text)


def format_unicode(s):
    """Converts a string in CIF text-format to unicode.  Any HTML tags
    contained in the string are removed.  HTML numeric character references
    are unescaped (i.e. converted to unicode).

    Parameters:

    s: string
        The CIF text string to convert

    Returns:

    u: string
        A unicode formatted string.
    """

    s = html.unescape(s)
    s = multiple_replace(s, subs_dict)
    tagclean = re.compile('<.*?>')
    return re.sub(tagclean, '', s)


def handle_subscripts(s):
    s = replace_subscript(s, subscript=True)
    s = replace_subscript(s, subscript=False)
    return s



"""This test cross-checks our implementation of CODATA against the
implementation that SciPy brings with it.
"""


def test_units():
    import numpy as np
    from ase.units import CODATA
    import scipy.constants.codata

    name_map = {'_c': 'speed of light in vacuum',
                '_mu0': 'mag. const.',
                '_Grav': 'Newtonian constant of gravitation',
                '_hplanck': 'Planck constant',
                '_e': 'elementary charge',
                '_me': 'electron mass',
                '_mp': 'proton mass',
                '_Nav': 'Avogadro constant',
                '_k': 'Boltzmann constant',
                '_amu': 'atomic mass unit-kilogram relationship'}

    for version in sorted(CODATA.keys()):
        print('Checking CODATA version "{0}"'.format(version))

        try:
            scipy_CODATA = getattr(scipy.constants.codata,
                                   '_physical_constants_{0}'.format(version))
        except AttributeError:
            print('\tNot available through scipy, skipping')
            continue

        for unit, scipyname in name_map.items():
            aseval = CODATA[version][unit]
            try:
                scipyval = scipy_CODATA[name_map[unit]][0]
                msg = 'Unit "{0}" : '.format(name_map[unit])
                ok = True
                if np.isclose(aseval, scipyval):
                    msg += '[OK]'
                else:
                    msg += '[FALSE]'
                    ok = False
                print('\t' + msg)
                if not ok:
                    raise AssertionError

            except KeyError:
                # 2002 in scipy contains too little data
                continue


def test_create_units():
    """Check that units are created and allow attribute access."""

    import ase.units

    print('Checking create_units and attribute access')
    # just use current CODATA version
    new_units = ase.units.create_units(ase.units.__codata_version__)
    assert new_units.eV == new_units['eV'] == ase.units.eV
    for unit_name in new_units.keys():
        assert getattr(new_units, unit_name) == getattr(ase.units, unit_name)
        assert new_units[unit_name] == getattr(ase.units, unit_name)


test_units()
test_create_units()



from ase.atoms import Atoms
from ase.calculators.singlepoint import SinglePointDFTCalculator
from ase.calculators.singlepoint import SinglePointKPoint
from ase.dft.dos import DOS


atoms = Atoms('H')
eFermi = [0, 1]
kpts = [SinglePointKPoint(1, 0, 0), SinglePointKPoint(1, 1, 0)]
kpts[0].eps_n = [-2, -1, 1]
kpts[0].f_n = [1, 0, 0]
kpts[1].eps_n = [-2.5, -1.5, 0.5]
kpts[1].f_n = [1, 0, 0]

calc = SinglePointDFTCalculator(atoms, efermi=eFermi)
calc.kpts = kpts

dos = DOS(calc)



""" Read/Write DL_POLY_4 CONFIG files """
import re

from numpy import zeros, isscalar

from ase.atoms import Atoms
from ase.data import chemical_symbols
from ase.calculators.singlepoint import SinglePointCalculator

__all__ = ['read_dlp4', 'write_dlp4']

# dlp4 labels will be registered in atoms.arrays[DLP4_LABELS_KEY]
DLP4_LABELS_KEY = 'dlp4_labels'

def _get_frame_positions(f):
    """Get positions of frames in HISTORY file."""
    #header line contains name of system
    init_pos = f.tell()
    f.seek(0)
    f.readline() #system name
    items = f.readline().strip().split()
    if len(items) == 5:
        classic = False
    elif len(items) == 3:
        classic = True
    else:
        raise RuntimeError("Cannot determine version of HISTORY file format.")

    levcfg,imcon,natoms = [int(x) for x in items[0:3]]
    if classic:
        #we have to iterate over the entire file
        startpos = f.tell()
        pos = []
        line = True
        while line:
            line = f.readline()
            if 'timestep' in line:
                pos.append(f.tell())
        f.seek(startpos)
    else:
        nFrames = int(line[4])
        pos = [(natoms * 2 + 1) * i + 2 for i in range(nFrames)]

    f.seek(init_pos)
    return levcfg,imcon,natoms,pos

def read_dlp_history(f, index=-1, symbols=None):
    """Read a HISTORY file.

    Compatible with DLP4 and DLP_CLASSIC.

    *Index* can be integer or slice.

    Provide a list of element strings to symbols to ignore naming from the HISTORY file.
    """
    levcfg,imcon,natoms,pos = _get_frame_positions(f)
    if isscalar(index):
        selected = [pos[index]]
    else:
        selected = pos[index]

    images = []
    for fpos in selected:
        f.seek(fpos+1)
        images.append(read_single_image(f, levcfg, imcon, natoms, is_trajectory=True, symbols=symbols))

    return images


def iread_dlp_history(f, symbols=None):
    """Generator version of read_history"""
    levcfg,imcon,natoms,pos = _get_frame_positions(f)
    for p in pos:
        f.seek(p+1)
        yield read_single_image(f, levcfg, imcon, natoms, is_trajectory=True, symbols=symbols)



def read_dlp4(f, symbols=None):
    """Read a DL_POLY_4 config/revcon file.

    Typically used indirectly through read('filename', atoms, format='dlp4').

    Can be unforgiven with custom chemical element names.
    Please complain to alin@elena.space for bugs."""

    line = f.readline()
    line = f.readline().split()
    levcfg = int(line[0])
    imcon = int(line[1])

    position = f.tell()
    line = f.readline()
    tokens = line.split()
    is_trajectory = tokens[0] == 'timestep'
    if not is_trajectory:
        # Difficult to distinguish between trajectory and non-trajectory
        # formats without reading one line too much.
        f.seek(position)

    while line:
        if is_trajectory:
            tokens = line.split()
            natoms = int(tokens[2])
        else:
            natoms = None
        yield read_single_image(f, levcfg, imcon, natoms, is_trajectory, symbols)
        line = f.readline()


def read_single_image(f, levcfg, imcon, natoms, is_trajectory, symbols=None):
    cell = zeros((3, 3))
    ispbc = imcon > 0
    if ispbc or is_trajectory:
        for j in range(3):
            line = f.readline()
            line = line.split()
            for i in range(3):
                try:
                    cell[j, i] = float(line[i])
                except ValueError:
                    raise RuntimeError("error reading cell")
    if symbols:
        sym = symbols
    else:
        sym = []
    positions = []
    velocities = []
    forces = []

    if is_trajectory:
        counter = range(natoms)
    else:
        from itertools import count
        counter = count()

    labels = []

    for a in counter:
        line = f.readline()
        if not line:
            a -= 1
            break

        m = re.match(r'\s*([A-Za-z][a-z]?)(\S*)', line)
        assert m is not None, line
        symbol, label = m.group(1, 2)
        symbol = symbol.capitalize()

        if not symbols:
            assert symbol in chemical_symbols
            sym.append(symbol)
        #make sure label is not empty
        if label:
            labels.append(label)
        else:
            labels.append(line.split()[0])


        x, y, z = f.readline().split()[:3]
        positions.append([float(x), float(y), float(z)])
        if levcfg > 0:
            vx, vy, vz = f.readline().split()[:3]
            velocities.append([float(vx), float(vy), float(vz)])
        if levcfg > 1:
            fx, fy, fz = f.readline().split()[:3]
            forces.append([float(fx), float(fy), float(fz)])

    if symbols:
        assert a+1 == len(symbols), ("Error, counter is at {:} but you gave {:} symbols".format(a+1,len(symbols)))

    if imcon == 0:
        pbc = False
    elif imcon == 6:
        pbc = [True, True, False]
    else:
        assert imcon in [1, 2, 3]
        pbc = True

    atoms = Atoms(positions=positions,
                  symbols=sym,
                  cell=cell,
                  pbc=pbc,
                  # Cell is centered around (0, 0, 0) in dlp4:
                  celldisp=-cell.sum(axis=0) / 2)

    atoms.set_array(DLP4_LABELS_KEY, labels, str)
    if levcfg > 0:
        atoms.set_velocities(velocities)
    if levcfg > 1:
        atoms.set_calculator(SinglePointCalculator(atoms, forces=forces))
    return atoms


def write_dlp4(f, atoms, levcfg=0, title='CONFIG generated by ASE'):
    """Write a DL_POLY_4 config file.

    Typically used indirectly through write('filename', atoms, format='dlp4').

    Can be unforgiven with custom chemical element names.
    Please complain to alin@elena.space in case of bugs"""

    f.write('{0:72s}\n'.format(title))
    natoms = len(atoms)
    npbc = sum(atoms.pbc)
    if npbc == 0:
        imcon = 0
    elif npbc == 3:
        imcon = 3
    elif tuple(atoms.pbc) == (True, True, False):
        imcon = 6
    else:
        raise ValueError('Unsupported pbc: {}.  '
                         'Supported pbc are 000, 110, and 000.'
                         .format(atoms.pbc))

    f.write('{0:10d}{1:10d}{2:10d}\n'.format(levcfg, imcon, natoms))
    if imcon > 0:
        cell = atoms.get_cell()
        for j in range(3):
            f.write('{0:20.10f}{1:20.10f}{2:20.10f}\n'.format(
                cell[j, 0], cell[j, 1], cell[j, 2]))
    vels = []
    forces = []
    if levcfg > 0:
        vels = atoms.get_velocities()
    if levcfg > 1:
        forces = atoms.get_forces()

    labels = atoms.arrays.get(DLP4_LABELS_KEY)

    for i, a in enumerate(atoms):
        sym = a.symbol
        if labels is not None:
            sym += labels[i]
        f.write("{0:8s}{1:10d}\n{2:20.10f}{3:20.10f}{4:20.10f}\n".format(
            sym, a.index+1, a.x, a.y, a.z))
        if levcfg > 0:
            if vels is None:
                f.write("{0:20.10f}{1:20.10f}{2:20.10f}\n".format(
                    0.0, 0.0, 0.0))
            else:
                f.write("{0:20.10f}{1:20.10f}{2:20.10f}\n".format(
                    vels[a.index, 0], vels[a.index, 1], vels[a.index, 2]))
        if levcfg > 1:
            if forces is None:
                f.write("{0:20.10f}{1:20.10f}{2:20.10f}\n".format(
                    0.0, 0.0, 0.0))
            else:
                f.write("{0:20.10f}{1:20.10f}{2:20.10f}\n".format(
                    forces[a.index, 0], forces[a.index, 1], forces[a.index, 2]))



from ase import Atoms
from gpaw import GPAW

atoms = Atoms('N')
atoms.center(vacuum=3.0)
atoms.set_initial_magnetic_moments([3])

calc = GPAW(mode='lcao', basis='dzp')
atoms.calc = calc
atoms.get_potential_energy()



import unittest
from matplotlib.animation import writers
import warnings

from ase.build import bulk, molecule, fcc111
from ase.io.animation import write_animation

if 'html' not in writers.list():
    raise unittest.SkipTest('matplotlib html writer not present')


images = [molecule('H2O'), bulk('Cu'), fcc111('Au', size=(1, 1, 1))]

# gif and mp4 writers may not be available.  Easiest solution is to only
# test this using the html writer because it always exists whenever
# matplotlib exists:
with warnings.catch_warnings():
    try:
        from matplotlib import MatplotlibDeprecationWarning
    except ImportError:
        pass
    else:
        warnings.simplefilter('ignore', MatplotlibDeprecationWarning)
    write_animation('things.html', images, writer='html')



# creates: WL.png, Ni111slab2x2.png, WL_rot_c.png, WL_rot_a.png, WL_wrap.png, interface-h2o-wrap.png
import numpy as np
from ase.io import read, write
from ase.build import fcc111

exec(compile(open('WL.py').read(), 'WL.py', 'exec'))

# Use ase.io.read to load atoms object
W = read('WL.traj')
# View the water unit or print the unit cell size.
write('WL.png', W)
# We will need cellW later.
cellW = W.get_cell()
print(cellW)

# We will need as close a lattice match as possible. lets try this slab.
# Using the ase.build module, we make the fcc111 slab.
slab = fcc111('Ni', size=[2, 4, 3], a=3.55, orthogonal=True)
cell = slab.get_cell()
write('Ni111slab2x2.png', slab)
print(cell)

# Rotate the unit cell first to get the close lattice match with the slab.
W.set_cell([[cellW[1, 1], 0, 0],
            [0, cellW[0, 0], 0],
            cellW[2]],
           scale_atoms=False)
write('WL_rot_c.png', W)

# Now rotate atoms just like the unit cell
W.rotate(90, 'z', center=(0, 0, 0))
write('WL_rot_a.png', W)

# Now we can use wrap
W.wrap()
write('WL_wrap.png', W)

# Match the water lattice to the slab by rescaling
cell1 = np.array([cell[0], cell[1], cellW[2]])
W.set_cell(cell1, scale_atoms=True)
# Set the positions of the water to be 1.5 aangstrom above the slab.
p = slab.get_positions()
W.center(vacuum=p[:, 2].max() + 1.5, axis=2)

# Finally use extend to combine the slab and waterlayer
interface = slab.copy()
interface.extend(W)
interface.center(vacuum=6, axis=2)
write('interface-h2o-wrap.png', interface)



import unittest

from ase.build import molecule
from ase import io
try:
    from gpaw import GPAW
except ImportError:
    raise unittest.SkipTest('GPAW not available')

txt = 'out.txt'
if 1:
    calculator = GPAW(h=0.3, txt=txt)
    atoms = molecule('H2', calculator=calculator)
    atoms.center(vacuum=3)
    atoms.get_potential_energy()
    atoms.set_initial_magnetic_moments([0.5, 0.5])
    calculator.set(charge=1)
    atoms.get_potential_energy()

# read again
t = io.read(txt, index=':')
assert isinstance(t, list)
M = t[1].get_magnetic_moments()
assert abs(M - 0.2).max() < 0.1



"""
This module contains functionality for reading and writing an ASE
Atoms object in VASP POSCAR format.

"""

import os
import re

import numpy as np

import ase.units
from ase import Atoms
from ase.utils import basestring, reader, writer
from ase.io.utils import ImageIterator, ImageChunk


__all__ = ['read_vasp', 'read_vasp_out', 'iread_vasp_out',
           'read_vasp_xdatcar', 'read_vasp_xml',
           'write_vasp']

# Denotes end of Ionic step for OUTCAR reading
_OUTCAR_SCF_DELIM = 'FREE ENERGIE OF THE ION-ELECTRON SYSTEM'


def get_atomtypes(fname):
    """Given a file name, get the atomic symbols.

    The function can get this information from OUTCAR and POTCAR
    format files.  The files can also be compressed with gzip or
    bzip2.

    """
    atomtypes = []
    if fname.find('.gz') != -1:
        import gzip
        f = gzip.open(fname)
    elif fname.find('.bz2') != -1:
        import bz2
        f = bz2.BZ2File(fname)
    else:
        f = open(fname)
    for line in f:
        if line.find('TITEL') != -1:
            atomtypes.append(line.split()[3].split('_')[0].split('.')[0])
    return atomtypes


def atomtypes_outpot(posfname, numsyms):
    """Try to retrieve chemical symbols from OUTCAR or POTCAR

    If getting atomtypes from the first line in POSCAR/CONTCAR fails, it might
    be possible to find the data in OUTCAR or POTCAR, if these files exist.

    posfname -- The filename of the POSCAR/CONTCAR file we're trying to read

    numsyms -- The number of symbols we must find

    """
    import os.path as op
    import glob

    # First check files with exactly same name except POTCAR/OUTCAR instead
    # of POSCAR/CONTCAR.
    fnames = [posfname.replace('POSCAR', 'POTCAR').replace('CONTCAR',
                                                           'POTCAR')]
    fnames.append(posfname.replace('POSCAR', 'OUTCAR').replace('CONTCAR',
                                                               'OUTCAR'))
    # Try the same but with compressed files
    fsc = []
    for fn in fnames:
        fsc.append(fn + '.gz')
        fsc.append(fn + '.bz2')
    for f in fsc:
        fnames.append(f)
    # Finally try anything with POTCAR or OUTCAR in the name
    vaspdir = op.dirname(posfname)
    fs = glob.glob(vaspdir + '*POTCAR*')
    for f in fs:
        fnames.append(f)
    fs = glob.glob(vaspdir + '*OUTCAR*')
    for f in fs:
        fnames.append(f)

    tried = []
    files_in_dir = os.listdir('.')
    for fn in fnames:
        if fn in files_in_dir:
            tried.append(fn)
            at = get_atomtypes(fn)
            if len(at) == numsyms:
                return at

    raise IOError('Could not determine chemical symbols. Tried files ' +
                  str(tried))


def get_atomtypes_from_formula(formula):
    """Return atom types from chemical formula (optionally prepended
    with and underscore).
    """
    from ase.symbols import string2symbols
    symbols = string2symbols(formula.split('_')[0])
    atomtypes = [symbols[0]]
    for s in symbols[1:]:
        if s != atomtypes[-1]:
            atomtypes.append(s)
    return atomtypes


@reader
def read_vasp(filename='CONTCAR'):
    """Import POSCAR/CONTCAR type file.

    Reads unitcell, atom positions and constraints from the POSCAR/CONTCAR
    file and tries to read atom types from POSCAR/CONTCAR header, if this fails
    the atom types are read from OUTCAR or POTCAR file.
    """

    from ase.constraints import FixAtoms, FixScaled
    from ase.data import chemical_symbols

    f = filename
    # The first line is in principle a comment line, however in VASP
    # 4.x a common convention is to have it contain the atom symbols,
    # eg. "Ag Ge" in the same order as later in the file (and POTCAR
    # for the full vasp run). In the VASP 5.x format this information
    # is found on the fifth line. Thus we save the first line and use
    # it in case we later detect that we're reading a VASP 4.x format
    # file.
    line1 = f.readline()

    lattice_constant = float(f.readline().split()[0])

    # Now the lattice vectors
    a = []
    for ii in range(3):
        s = f.readline().split()
        floatvect = float(s[0]), float(s[1]), float(s[2])
        a.append(floatvect)

    basis_vectors = np.array(a) * lattice_constant

    # Number of atoms. Again this must be in the same order as
    # in the first line
    # or in the POTCAR or OUTCAR file
    atom_symbols = []
    numofatoms = f.readline().split()
    # Check whether we have a VASP 4.x or 5.x format file. If the
    # format is 5.x, use the fifth line to provide information about
    # the atomic symbols.
    vasp5 = False
    try:
        int(numofatoms[0])
    except ValueError:
        vasp5 = True
        atomtypes = numofatoms
        numofatoms = f.readline().split()

    # check for comments in numofatoms line and get rid of them if necessary
    commentcheck = np.array(['!' in s for s in numofatoms])
    if commentcheck.any():
        # only keep the elements up to the first including a '!':
        numofatoms = numofatoms[:np.arange(len(numofatoms))[commentcheck][0]]

    if not vasp5:
        # Split the comment line (first in the file) into words and 
        # try to compose a list of chemical symbols
        from ase.formula import Formula
        import re
        atomtypes = []
        for word in line1.split():
            word_without_delims = re.sub(r"-|_|,|\.|=|[0-9]|^", "", word)
            if len(word_without_delims) < 1:
                continue
            try:
                atomtypes.extend(list(Formula(word_without_delims)))
            except ValueError:
                #print(atomtype, e, 'is comment')
                pass
        # Now the list of chemical symbols atomtypes must be formed.
        # For example: atomtypes = ['Pd', 'C', 'O']
        
        numsyms = len(numofatoms)
        if len(atomtypes) < numsyms:
            # First line in POSCAR/CONTCAR didn't contain enough symbols.

            # Sometimes the first line in POSCAR/CONTCAR is of the form
            # "CoP3_In-3.pos". Check for this case and extract atom types
            if len(atomtypes) == 1 and '_' in atomtypes[0]:
                atomtypes = get_atomtypes_from_formula(atomtypes[0])
            else:
                atomtypes = atomtypes_outpot(f.name, numsyms)
        else:
            try:
                for atype in atomtypes[:numsyms]:
                    if atype not in chemical_symbols:
                        raise KeyError
            except KeyError:
                atomtypes = atomtypes_outpot(f.name, numsyms)

    for i, num in enumerate(numofatoms):
        numofatoms[i] = int(num)
        [atom_symbols.append(atomtypes[i]) for na in range(numofatoms[i])]

    # Check if Selective dynamics is switched on
    sdyn = f.readline()
    selective_dynamics = sdyn[0].lower() == 's'

    # Check if atom coordinates are cartesian or direct
    if selective_dynamics:
        ac_type = f.readline()
    else:
        ac_type = sdyn
    cartesian = ac_type[0].lower() == 'c' or ac_type[0].lower() == 'k'
    tot_natoms = sum(numofatoms)
    atoms_pos = np.empty((tot_natoms, 3))
    if selective_dynamics:
        selective_flags = np.empty((tot_natoms, 3), dtype=bool)
    for atom in range(tot_natoms):
        ac = f.readline().split()
        atoms_pos[atom] = (float(ac[0]), float(ac[1]), float(ac[2]))
        if selective_dynamics:
            curflag = []
            for flag in ac[3:6]:
                curflag.append(flag == 'F')
            selective_flags[atom] = curflag
    if cartesian:
        atoms_pos *= lattice_constant
    atoms = Atoms(symbols=atom_symbols, cell=basis_vectors, pbc=True)
    if cartesian:
        atoms.set_positions(atoms_pos)
    else:
        atoms.set_scaled_positions(atoms_pos)
    if selective_dynamics:
        constraints = []
        indices = []
        for ind, sflags in enumerate(selective_flags):
            if sflags.any() and not sflags.all():
                constraints.append(FixScaled(atoms.get_cell(), ind, sflags))
            elif sflags.all():
                indices.append(ind)
        if indices:
            constraints.append(FixAtoms(indices))
        if constraints:
            atoms.set_constraint(constraints)
    return atoms


class OUTCARChunk(ImageChunk):
    def __init__(self, lines, header_data):
        self.lines = lines
        self.header_data = header_data

    def build(self):
        return _read_outcar_frame(self.lines, self.header_data)


def _read_outcar_frame(lines, header_data):
    from ase.calculators.singlepoint import (SinglePointDFTCalculator,
                                             SinglePointKPoint)

    mag_x = None
    mag_y = None
    mag_z = None
    magmoms = None
    magmom = None
    stress = None
    efermi = None

    symbols = header_data['symbols']
    constraints = header_data['constraints']
    natoms = header_data['natoms']
    # nkpts = header_data['nkpts']
    nbands = header_data['nbands']
    kpt_weights = header_data['kpt_weights']
    ibzkpts = header_data['ibzkpts']

    atoms = Atoms(symbols=symbols, pbc=True, constraint=constraints)

    cl = _outcar_check_line     # Aliasing

    spinc = 0                   # Spin component
    kpts = []
    forces = np.zeros((natoms, 3))
    positions = np.zeros((natoms, 3))
    f_n = np.zeros(nbands)      # kpt occupations
    eps_n = np.zeros(nbands)    # kpt eigenvalues

    # Parse each atoms object
    for n, line in enumerate(lines):
        line = line.strip()
        if 'direct lattice vectors' in line:
            cell = []
            for i in range(3):
                parts = cl(lines[n + i + 1]).split()
                cell += [list(map(float, parts[0:3]))]
            atoms.set_cell(cell)
        elif 'magnetization (x)' in line:
            # Magnetization in both collinear and non-collinear
            nskip = 4           # Skip some lines
            mag_x = [float(cl(lines[n + i + nskip]).split()[-1])
                     for i in range(natoms)]

        # XXX: !!!Uncomment these lines when non-collinear spin is supported!!!
        # Remember to check that format fits!

        # elif 'magnetization (y)' in line:
        #     # Non-collinear spin
        #     nskip = 4           # Skip some lines
        #     mag_y = [float(cl(lines[n + i + nskip]).split()[-1])
        #              for i in range(natoms)]
        # elif 'magnetization (z)' in line:
        #     # Non-collinear spin
        #     nskip = 4           # Skip some lines
        #     mag_z = [float(cl(lines[n + i + nskip]).split()[-1])
        #              for i in range(natoms)]
        elif 'number of electron' in line:
            parts = cl(line).split()
            if len(parts) > 5 and parts[0].strip() != "NELECT":
                i = parts.index('magnetization') + 1
                magmom = parts[i:]
                if len(magmom) == 1:
                    # Collinear spin
                    magmom = float(magmom[0])
                # XXX: !!!Uncomment these lines when non-collinear spin is supported!!!
                # Remember to check that format fits!
                # else:
                #     # Non-collinear spin
                #     # Make a (3,) dim array
                #     magmom = np.array(list(map(float, magmom)))
        elif 'in kB ' in line:
            stress = -np.asarray([float(a) for a in cl(line).split()[2:]])
            stress = stress[[0, 1, 2, 4, 5, 3]] * 1e-1 * ase.units.GPa
        elif 'POSITION          ' in line:
            nskip = 2
            for i in range(natoms):
                parts = list(map(float, cl(lines[n + i + nskip]).split()))
                positions[i] = parts[0:3]
                forces[i] = parts[3:6]
            atoms.set_positions(positions, apply_constraint=False)
        elif 'E-fermi :' in line:
            parts = line.split()
            efermi = float(parts[2])
        elif 'spin component' in line:
            # Update spin component for kpts
            # Make spin be in [0, 1], VASP writes 1 or 2
            tmp = int(line.split()[-1]) - 1
            if tmp < spinc:
                # if NWRITE=3, we write KPTS after every electronic step,
                # so we just reset it, since we went from spin=2 to spin=1
                # in the same ionic step.
                # XXX: Only read it at last electronic step
                kpts = []
            spinc = tmp
        elif 'k-point  ' in line:
            if 'plane waves' in line:
                # Can happen if we still have part of header
                continue
            # Parse all kpts and bands
            parts = line.split()
            ikpt = int(parts[1]) - 1  # Make kpt idx start from 0
            w = kpt_weights[ikpt]

            nskip = 2
            for i in range(nbands):
                parts = lines[n + i + nskip].split()
                eps_n[i] = float(parts[1])
                f_n[i] = float(parts[2])
            kpts.append(SinglePointKPoint(w, spinc, ikpt,
                                          eps_n=eps_n, f_n=f_n))
        elif _OUTCAR_SCF_DELIM in line:
            # Last section before next ionic step
            nskip = 2
            parts = cl(lines[n + nskip]).strip().split()
            energy_free = float(parts[4])  # Force consistent

            nskip = 4
            parts = cl(lines[n + nskip]).strip().split()
            energy_zero = float(parts[6])  # Extrapolated to 0 K

            # For debugging
            # assert len(kpts) == 0 or len(kpts) == (spinc + 1) * nkpts

            if mag_x is not None:
                if mag_y is not None:
                    # Non-collinear
                    assert len(mag_x) == len(mag_y) == len(mag_z)
                    magmoms = np.zeros((len(atoms), 3))
                    magmoms[:, 0] = mag_x
                    magmoms[:, 1] = mag_y
                    magmoms[:, 2] = mag_z
                else:
                    # Collinear
                    magmoms = np.array(mag_x)

            atoms.set_calculator(
                SinglePointDFTCalculator(atoms,
                                         energy=energy_zero,
                                         free_energy=energy_free,
                                         ibzkpts=ibzkpts,
                                         forces=forces,
                                         efermi=efermi,
                                         magmom=magmom,
                                         magmoms=magmoms,
                                         stress=stress))
            atoms.calc.name = 'vasp'
            atoms.calc.kpts = kpts
    return atoms


def _outcar_check_line(line):
    """Auxiliary check line function for OUTCAR numeric formatting.
    See issue #179, https://gitlab.com/ase/ase/issues/179
    Only call in cases we need the numeric values
    """
    if re.search('[0-9]-[0-9]', line):
        line = re.sub('([0-9])-([0-9])', r'\1 -\2', line)
    return line


def _read_outcar_header(fd):

    # Get the directory of the OUTCAR we are reading
    wdir = os.path.dirname(fd.name)
    # Try and see if we can get constraints
    if os.path.isfile(os.path.join(wdir, 'CONTCAR')):
        constraints = read_vasp(os.path.join(wdir, 'CONTCAR')).constraints
    elif os.path.isfile(os.path.join(wdir, 'POSCAR')):
        constraints = read_vasp(os.path.join(wdir, 'POSCAR')).constraints
    else:
        constraints = None

    cl = _outcar_check_line     # Aliasing

    species = []
    natoms = 0
    species_num = []
    symbols = []
    nkpts = 0
    nbands = 0
    kpt_weights = []
    ibzkpts = []

    # Get atomic species
    for line in fd:
        line = line.strip()
        if 'POTCAR:' in line:
            temp = line.split()[2]
            for c in ['.', '_', '1']:
                if c in temp:
                    temp = temp[0:temp.find(c)]
            species += [temp]
        elif 'ions per type' in line:
            species = species[:len(species) // 2]
            parts = cl(line).split()
            ntypes = min(len(parts) - 4, len(species))
            for ispecies in range(ntypes):
                species_num += [int(parts[ispecies + 4])]
                natoms += species_num[-1]
                for iatom in range(species_num[-1]):
                    symbols += [species[ispecies]]
        elif 'NKPTS' in line:
            parts = cl(line).split()
            nkpts = int(parts[3])
            nbands = int(parts[-1])
        elif 'k-points in reciprocal lattice and weights' in line:
            # Get kpoint weights
            for _ in range(nkpts):
                parts = next(fd).strip().split()
                ibzkpts.append(list(map(float, parts[0:3])))
                kpt_weights.append(float(parts[-1]))

        elif 'Iteration' in line:
            # Start of SCF cycle
            header_data = dict(
                natoms=natoms,
                symbols=symbols,
                constraints=constraints,
                nkpts=nkpts,
                nbands=nbands,
                kpt_weights=np.array(kpt_weights),
                ibzkpts=np.array(ibzkpts))
            return header_data

    # Incomplete OUTCAR, we can't determine atoms
    raise IOError('Incomplete OUTCAR')


def outcarchunks(fd):
    # First we get header info
    header_data = _read_outcar_header(fd)

    while True:
        try:
            # Build chunk which contains 1 complete atoms object
            lines = []
            while True:
                line = next(fd)
                lines.append(line)
                if _OUTCAR_SCF_DELIM in line:
                    # Add 4 more lines to include energy
                    for _ in range(4):
                        lines.append(next(fd))
                    break
        except StopIteration:
            # End of file
            return
        yield OUTCARChunk(lines, header_data)


def iread_vasp_out(filename, index=-1):
    """Import OUTCAR type file, as a generator."""
    it = ImageIterator(outcarchunks)
    return it(filename, index=index)


@reader
def read_vasp_out(filename='OUTCAR', index=-1):
    """Import OUTCAR type file.

    Reads unitcell, atom positions, energies, and forces from the OUTCAR file
    and attempts to read constraints (if any) from CONTCAR/POSCAR, if present.
    """
    f = filename
    g = iread_vasp_out(f, index=index)
    # Code borrowed from formats.py:read
    if isinstance(index, (slice, basestring)):
        # Return list of atoms
        return list(g)
    else:
        # Return single atoms object
        return next(g)


@reader
def read_vasp_xdatcar(filename='XDATCAR', index=-1):
    """Import XDATCAR file

       Reads all positions from the XDATCAR and returns a list of
       Atoms objects.  Useful for viewing optimizations runs
       from VASP5.x

       Constraints ARE NOT stored in the XDATCAR, and as such, Atoms
       objects retrieved from the XDATCAR will not have constraints set.
    """
    f = filename
    images = list()

    cell = np.eye(3)
    atomic_formula = str()

    while True:
        comment_line = f.readline()
        if "Direct configuration=" not in comment_line:
            try:
                lattice_constant = float(f.readline())
            except Exception:
                # XXX: When would this happen?
                break

            xx = [float(x) for x in f.readline().split()]
            yy = [float(y) for y in f.readline().split()]
            zz = [float(z) for z in f.readline().split()]
            cell = np.array([xx, yy, zz]) * lattice_constant

            symbols = f.readline().split()
            numbers = [int(n) for n in f.readline().split()]
            total = sum(numbers)

            atomic_formula = ''.join('{:s}{:d}'.format(sym, numbers[n])
                                     for n, sym in enumerate(symbols))

            f.readline()

        coords = [np.array(f.readline().split(), np.float)
                  for ii in range(total)]

        image = Atoms(atomic_formula, cell=cell, pbc=True)
        image.set_scaled_positions(np.array(coords))
        images.append(image)

    if not index:
        return images
    else:
        return images[index]


def __get_xml_parameter(par):
    """An auxiliary function that enables convenient extraction of
    parameter values from a vasprun.xml file with proper type
    handling.

    """

    def to_bool(b):
        if b == 'T':
            return True
        else:
            return False

    to_type = {'int': int,
               'logical': to_bool,
               'string': str,
               'float': float}

    text = par.text
    if text is None:
        text = ''

    # Float parameters do not have a 'type' attrib
    var_type = to_type[par.attrib.get('type', 'float')]

    try:
        if par.tag == 'v':
            return list(map(var_type, text.split()))
        else:
            return var_type(text.strip())
    except ValueError:
        # Vasp can sometimes write "*****" due to overflow
        return None


def read_vasp_xml(filename='vasprun.xml', index=-1):
    """Parse vasprun.xml file.

    Reads unit cell, atom positions, energies, forces, and constraints
    from vasprun.xml file
    """

    import xml.etree.ElementTree as ET
    from ase.constraints import FixAtoms, FixScaled
    from ase.calculators.singlepoint import (SinglePointDFTCalculator,
                                             SinglePointKPoint)
    from ase.units import GPa
    from collections import OrderedDict

    tree = ET.iterparse(filename, events=['start', 'end'])

    atoms_init = None
    calculation = []
    ibz_kpts = None
    kpt_weights = None
    parameters = OrderedDict()

    try:
        for event, elem in tree:

            if event == 'end':
                if elem.tag == 'kpoints':
                    for subelem in elem.iter(tag='generation'):
                        kpts_params = OrderedDict()
                        parameters['kpoints_generation'] = kpts_params
                        for par in subelem.iter():
                            if par.tag in ['v', 'i']:
                                parname = par.attrib['name'].lower()
                                kpts_params[parname] = __get_xml_parameter(par)

                    kpts = elem.findall("varray[@name='kpointlist']/v")
                    ibz_kpts = np.zeros((len(kpts), 3))

                    for i, kpt in enumerate(kpts):
                        ibz_kpts[i] = [float(val) for val in kpt.text.split()]

                    kpt_weights = elem.findall('varray[@name="weights"]/v')
                    kpt_weights = [float(val.text) for val in kpt_weights]

                elif elem.tag == 'parameters':
                    for par in elem.iter():
                        if par.tag in ['v', 'i']:
                            parname = par.attrib['name'].lower()
                            parameters[parname] = __get_xml_parameter(par)

                elif elem.tag == 'atominfo':
                    species = []

                    for entry in elem.find("array[@name='atoms']/set"):
                        species.append(entry[0].text.strip())

                    natoms = len(species)

                elif (elem.tag == 'structure' and
                      elem.attrib.get('name') == 'initialpos'):
                    cell_init = np.zeros((3, 3), dtype=float)

                    for i, v in enumerate(elem.find(
                            "crystal/varray[@name='basis']")):
                        cell_init[i] = np.array([
                            float(val) for val in v.text.split()])

                    scpos_init = np.zeros((natoms, 3), dtype=float)

                    for i, v in enumerate(elem.find(
                            "varray[@name='positions']")):
                        scpos_init[i] = np.array([
                            float(val) for val in v.text.split()])

                    constraints = []
                    fixed_indices = []

                    for i, entry in enumerate(elem.findall(
                            "varray[@name='selective']/v")):
                        flags = (np.array(entry.text.split() ==
                                          np.array(['F', 'F', 'F'])))
                        if flags.all():
                            fixed_indices.append(i)
                        elif flags.any():
                            constraints.append(FixScaled(cell_init, i, flags))

                    if fixed_indices:
                        constraints.append(FixAtoms(fixed_indices))

                    atoms_init = Atoms(species,
                                       cell=cell_init,
                                       scaled_positions=scpos_init,
                                       constraint=constraints,
                                       pbc=True)

                elif elem.tag == 'dipole':
                    dblock = elem.find('v[@name="dipole"]')
                    if dblock is not None:
                        dipole = np.array([float(val)
                                           for val in dblock.text.split()])

            elif event == 'start' and elem.tag == 'calculation':
                calculation.append(elem)

    except ET.ParseError as parse_error:
        if atoms_init is None:
            raise parse_error
        if calculation[-1].find('energy') is None:
            calculation = calculation[:-1]
        if not calculation:
            yield atoms_init

    if calculation:
        if isinstance(index, int):
            steps = [calculation[index]]
        else:
            steps = calculation[index]
    else:
        steps = []

    for step in steps:
        # Workaround for VASP bug, e_0_energy contains the wrong value
        # in calculation/energy, but calculation/scstep/energy does not
        # include classical VDW corrections. So, first calculate
        # e_0_energy - e_fr_energy from calculation/scstep/energy, then
        # apply that correction to e_fr_energy from calculation/energy.
        lastscf = step.findall('scstep/energy')[-1]
        try:
            lastdipole = step.findall('scstep/dipole')[-1]
        except:
            lastdipole = None

        de = (float(lastscf.find('i[@name="e_0_energy"]').text) -
              float(lastscf.find('i[@name="e_fr_energy"]').text))

        free_energy = float(step.find('energy/i[@name="e_fr_energy"]').text)
        energy = free_energy + de

        cell = np.zeros((3, 3), dtype=float)
        for i, vector in enumerate(step.find(
                'structure/crystal/varray[@name="basis"]')):
            cell[i] = np.array([float(val) for val in vector.text.split()])

        scpos = np.zeros((natoms, 3), dtype=float)
        for i, vector in enumerate(step.find(
                'structure/varray[@name="positions"]')):
            scpos[i] = np.array([float(val) for val in vector.text.split()])

        forces = None
        fblocks = step.find('varray[@name="forces"]')
        if fblocks is not None:
            forces = np.zeros((natoms, 3), dtype=float)
            for i, vector in enumerate(fblocks):
                forces[i] = np.array([float(val)
                                      for val in vector.text.split()])

        stress = None
        sblocks = step.find('varray[@name="stress"]')
        if sblocks is not None:
            stress = np.zeros((3, 3), dtype=float)
            for i, vector in enumerate(sblocks):
                stress[i] = np.array([float(val)
                                      for val in vector.text.split()])
            stress *= -0.1 * GPa
            stress = stress.reshape(9)[[0, 4, 8, 5, 2, 1]]

        dipole = None
        if lastdipole is not None:
            dblock = lastdipole.find('v[@name="dipole"]')
            if dblock is not None:
                dipole = np.zeros((1, 3), dtype=float)
                dipole = np.array([float(val) for val in dblock.text.split()])

        dblock = step.find('dipole/v[@name="dipole"]')
        if dblock is not None:
            dipole = np.zeros((1, 3), dtype=float)
            dipole = np.array([float(val) for val in dblock.text.split()])

        efermi = step.find('dos/i[@name="efermi"]')
        if efermi is not None:
            efermi = float(efermi.text)

        kpoints = []
        for ikpt in range(1, len(ibz_kpts) + 1):
            kblocks = step.findall(
                'eigenvalues/array/set/set/set[@comment="kpoint %d"]' % ikpt)
            if kblocks is not None:
                for spin, kpoint in enumerate(kblocks):
                    eigenvals = kpoint.findall('r')
                    eps_n = np.zeros(len(eigenvals))
                    f_n = np.zeros(len(eigenvals))
                    for j, val in enumerate(eigenvals):
                        val = val.text.split()
                        eps_n[j] = float(val[0])
                        f_n[j] = float(val[1])
                    if len(kblocks) == 1:
                        f_n *= 2
                    kpoints.append(SinglePointKPoint(kpt_weights[ikpt - 1],
                                                     spin, ikpt, eps_n, f_n))
        if len(kpoints) == 0:
            kpoints = None

        atoms = atoms_init.copy()
        atoms.set_cell(cell)
        atoms.set_scaled_positions(scpos)
        atoms.set_calculator(
            SinglePointDFTCalculator(atoms, energy=energy, forces=forces,
                                     stress=stress, free_energy=free_energy,
                                     ibzkpts=ibz_kpts,
                                     efermi=efermi, dipole=dipole))
        atoms.calc.name = 'vasp'
        atoms.calc.kpts = kpoints
        atoms.calc.parameters = parameters
        yield atoms


@writer
def write_vasp(filename, atoms, label='', direct=False, sort=None,
               symbol_count=None, long_format=True, vasp5=False,
               ignore_constraints=False):
    """Method to write VASP position (POSCAR/CONTCAR) files.

    Writes label, scalefactor, unitcell, # of various kinds of atoms,
    positions in cartesian or scaled coordinates (Direct), and constraints
    to file. Cartesian coordiantes is default and default label is the
    atomic species, e.g. 'C N H Cu'.
    """

    from ase.constraints import FixAtoms, FixScaled, FixedPlane, FixedLine

    f = filename

    if isinstance(atoms, (list, tuple)):
        if len(atoms) > 1:
            raise RuntimeError('Don\'t know how to save more than ' +
                               'one image to VASP input')
        else:
            atoms = atoms[0]

    # Check lattice vectors are finite
    if np.any(atoms.get_cell_lengths_and_angles() == 0.):
        raise RuntimeError(
            'Lattice vectors must be finite and not coincident. '
            'At least one lattice length or angle is zero.')

    # Write atom positions in scaled or cartesian coordinates
    if direct:
        coord = atoms.get_scaled_positions()
    else:
        coord = atoms.get_positions()

    constraints = atoms.constraints and not ignore_constraints

    if constraints:
        sflags = np.zeros((len(atoms), 3), dtype=bool)
        for constr in atoms.constraints:
            if isinstance(constr, FixScaled):
                sflags[constr.a] = constr.mask
            elif isinstance(constr, FixAtoms):
                sflags[constr.index] = [True, True, True]
            elif isinstance(constr, FixedPlane):
                mask = np.all(np.abs(np.cross(constr.dir, atoms.cell)) < 1e-5,
                              axis=1)
                if sum(mask) != 1:
                    raise RuntimeError(
                        'VASP requires that the direction of FixedPlane '
                        'constraints is parallel with one of the cell axis')
                sflags[constr.a] = mask
            elif isinstance(constr, FixedLine):
                mask = np.all(np.abs(np.cross(constr.dir, atoms.cell)) < 1e-5,
                              axis=1)
                if sum(mask) != 1:
                    raise RuntimeError(
                        'VASP requires that the direction of FixedLine '
                        'constraints is parallel with one of the cell axis')
                sflags[constr.a] = ~mask

    if sort:
        ind = np.argsort(atoms.get_chemical_symbols())
        symbols = np.array(atoms.get_chemical_symbols())[ind]
        coord = coord[ind]
        if constraints:
            sflags = sflags[ind]
    else:
        symbols = atoms.get_chemical_symbols()

    # Create a list sc of (symbol, count) pairs
    if symbol_count:
        sc = symbol_count
    else:
        sc = []
        psym = symbols[0]
        count = 0
        for sym in symbols:
            if sym != psym:
                sc.append((psym, count))
                psym = sym
                count = 1
            else:
                count += 1
        sc.append((psym, count))

    # Create the label
    if label == '':
        for sym, c in sc:
            label += '%2s ' % sym
    f.write(label + '\n')

    # Write unitcell in real coordinates and adapt to VASP convention
    # for unit cell
    # ase Atoms doesn't store the lattice constant separately, so always
    # write 1.0.
    f.write('%19.16f\n' % 1.0)
    if long_format:
        latt_form = ' %21.16f'
    else:
        latt_form = ' %11.6f'
    for vec in atoms.get_cell():
        f.write(' ')
        for el in vec:
            f.write(latt_form % el)
        f.write('\n')

    # If we're writing a VASP 5.x format POSCAR file, write out the
    # atomic symbols
    if vasp5:
        for sym, c in sc:
            f.write(' %3s' % sym)
        f.write('\n')

    # Numbers of each atom
    for sym, count in sc:
        f.write(' %3i' % count)
    f.write('\n')

    if constraints:
        f.write('Selective dynamics\n')

    if direct:
        f.write('Direct\n')
    else:
        f.write('Cartesian\n')

    if long_format:
        cform = ' %19.16f'
    else:
        cform = ' %9.6f'
    for iatom, atom in enumerate(coord):
        for dcoord in atom:
            f.write(cform % dcoord)
        if constraints:
            for flag in sflags[iatom]:
                if flag:
                    s = 'F'
                else:
                    s = 'T'
                f.write('%4s' % s)
        f.write('\n')



import os
import numpy as np
from copy import deepcopy

from ase.calculators.calculator import KPoints, kpts2kpts

_special_kws = ['center', 'autosym', 'autoz', 'theory', 'basis', 'xc', 'task',
                'set', 'symmetry', 'label', 'geompar', 'basispar', 'kpts',
                'bandpath']

_system_type = {1: 'polymer', 2: 'surface', 3: 'crystal'}


def _get_geom(atoms, **params):
    geom_header = ['geometry units angstrom']
    for geomkw in ['center', 'autosym', 'autoz']:
        geom_header.append(geomkw if params.get(geomkw) else 'no' + geomkw)
    if 'geompar' in params:
        geom_header.append(params['geompar'])
    geom = [' '.join(geom_header)]

    outpos = atoms.get_positions()
    pbc = atoms.pbc
    if np.any(pbc):
        scpos = atoms.get_scaled_positions()
        for i, pbci in enumerate(pbc):
            if pbci:
                outpos[:, i] = scpos[:, i]
        npbc = pbc.sum()
        cellpars = atoms.get_cell_lengths_and_angles()
        geom.append('  system {} units angstrom'.format(_system_type[npbc]))
        if npbc == 3:
            geom.append('    lattice_vectors')
            for row in atoms.cell:
                geom.append('      {:20.16e} {:20.16e} {:20.16e}'.format(*row))
        else:
            if pbc[0]:
                geom.append('    lat_a {:20.16e}'.format(cellpars[0]))
            if pbc[1]:
                geom.append('    lat_b {:20.16e}'.format(cellpars[1]))
            if pbc[2]:
                geom.append('    lat_c {:20.16e}'.format(cellpars[2]))
            if pbc[1] and pbc[2]:
                geom.append('    alpha {:20.16e}'.format(cellpars[3]))
            if pbc[0] and pbc[2]:
                geom.append('    beta {:20.16e}'.format(cellpars[4]))
            if pbc[1] and pbc[0]:
                geom.append('    gamma {:20.16e}'.format(cellpars[5]))
        geom.append('  end')

    for i, atom in enumerate(atoms):
        geom.append('  {:<2} {:20.16e} {:20.16e} {:20.16e}'
                    ''.format(atom.symbol, *outpos[i]))
    symm = params.get('symmetry')
    if symm is not None:
        geom.append('  symmetry {}'.format(symm))
    geom.append('end')
    return geom


def _get_basis(theory, **params):
    if 'basis' not in params:
        if theory in ['pspw', 'band', 'paw']:
            return []
    basis_in = params.get('basis', '3-21G')
    if 'basispar' in params:
        header = 'basis {} noprint'.format(params['basispar'])
    else:
        header = 'basis noprint'
    basis_out = [header]
    if isinstance(basis_in, str):
        basis_out.append('   * library {}'.format(basis_in))
    else:
        for symbol, ibasis in basis_in.items():
            basis_out.append('{:>4} library {}'.format(symbol, ibasis))
    basis_out.append('end')
    return basis_out


_special_keypairs = [('nwpw', 'simulation_cell'),
                     ('nwpw', 'carr-parinello'),
                     ('nwpw', 'brillouin_zone'),
                     ('tddft', 'grad'),
                     ]


def _format_brillouin_zone(array, name=None):
    out = ['  brillouin_zone']
    if name is not None:
        out += ['    zone_name {}'.format(name)]
    template = '    kvector' + ' {:20.16e}' * array.shape[1]
    for row in array:
        out.append(template.format(*row))
    out.append('  end')
    return out


def _get_bandpath(bp):
    if bp is None:
        return []
    out = ['nwpw']
    out += _format_brillouin_zone(bp.kpts, name=bp.path)
    out += ['  zone_structure_name {}'.format(bp.path),
            'end',
            'task band structure']
    return out


def _format_line(key, val):
    if val is None:
        return key
    if isinstance(val, bool):
        return '{} .{}.'.format(key, str(val).lower())
    else:
        return ' '.join([key, str(val)])


def _format_block(key, val, nindent=0):
    prefix = '  ' * nindent
    prefix2 = '  ' * (nindent + 1)
    if val is None:
        return [prefix + key]

    if not isinstance(val, dict):
        return [prefix + _format_line(key, val)]

    out = [prefix + key]
    for subkey, subval in val.items():
        if (key, subkey) in _special_keypairs:
            if (key, subkey) == ('nwpw', 'brillouin_zone'):
                out += _format_brillouin_zone(subval)
            else:
                out += _format_block(subkey, subval, nindent + 1)
        else:
            if isinstance(subval, dict):
                subval = ' '.join([_format_line(a, b)
                                   for a, b in subval.items()])
            out.append(prefix2 + ' '.join([subkey, str(subval)]))
    out.append(prefix + 'end')
    return out


def _get_other(**params):
    out = []
    for kw, block in params.items():
        if kw in _special_kws:
            continue
        out += _format_block(kw, block)
    return out


def _get_set(**params):
    return ['set ' + _format_line(key, val) for key, val in params.items()]


_gto_theories = ['tce', 'ccsd', 'mp2', 'tddft', 'scf', 'dft']
_pw_theories = ['band', 'pspw', 'paw']
_all_theories = _gto_theories + _pw_theories


def _get_theory(**params):
    # Default: user-provided theory
    theory = params.get('theory')
    if theory is not None:
        return theory

    # Check if the user passed a theory to xc
    xc = params.get('xc')
    if xc in _all_theories:
        return xc

    # Check for input blocks that correspond to a particular level of
    # theory. Correlated theories (e.g. CCSD) are checked first.
    for kw in _gto_theories:
        if kw in params:
            return kw

    # If the user passed an 'nwpw' block, then they want a plane-wave
    # calculation, but what kind? If they request k-points, then
    # they want 'band', otherwise assume 'pspw' (if the user wants
    # to use 'paw', they will have to ask for it specifically).
    nwpw = params.get('nwpw')
    if nwpw is not None:
        if 'monkhorst-pack' in nwpw or 'brillouin_zone' in nwpw:
            return 'band'
        return 'pspw'

    # When all else fails, default to dft.
    return 'dft'


_xc_conv = dict(lda='slater pw91lda',
                pbe='xpbe96 cpbe96',
                revpbe='revpbe cpbe96',
                rpbe='rpbe cpbe96',
                pw91='xperdew91 perdew91',
                )


def _update_mult(magmom_tot, **params):
    theory = params['theory']
    if magmom_tot == 0:
        magmom_mult = 1
    else:
        magmom_mult = np.sign(magmom_tot) * (abs(magmom_tot) + 1)
    if 'scf' in params:
        for kw in ['nopen', 'singlet', 'doublet', 'triplet', 'quartet',
                   'quintet', 'sextet', 'septet', 'octet']:
            if kw in params['scf']:
                break
        else:
            params['scf']['nopen'] = magmom_tot
    elif theory in ['scf', 'mp2', 'ccsd', 'tce']:
        params['scf'] = dict(nopen=magmom_tot)

    if 'dft' in params:
        if 'mult' not in params['dft']:
            params['dft']['mult'] = magmom_mult
    elif theory in ['dft', 'tddft']:
        params['dft'] = dict(mult=magmom_mult)

    if 'nwpw' in params:
        if 'mult' not in params['nwpw']:
            params['nwpw']['mult'] = magmom_mult
    elif theory in ['pspw', 'band', 'paw']:
        params['nwpw'] = dict(mult=magmom_mult)

    return params


def _get_kpts(atoms, **params):
    """Converts top-level 'kpts' argument to native keywords"""
    kpts = params.get('kpts')
    if kpts is None:
        return params

    nwpw = params.get('nwpw', dict())

    if 'monkhorst-pack' in nwpw or 'brillouin_zone' in nwpw:
        raise ValueError("Redundant k-points specified!")

    if isinstance(kpts, KPoints):
        nwpw['brillouin_zone'] = kpts.kpts
    elif isinstance(kpts, dict):
        if kpts.get('gamma', False) or 'size' not in kpts:
            nwpw['brillouin_zone'] = kpts2kpts(kpts, atoms).kpts
        else:
            nwpw['monkhorst-pack'] = ' '.join(map(str, kpts['size']))
    elif isinstance(kpts, np.ndarray):
        nwpw['brillouin_zone'] = kpts
    else:
        nwpw['monkhorst-pack'] = ' '.join(map(str, kpts))

    params['nwpw'] = nwpw
    return params


def write_nwchem_in(fd, atoms, properties=None, **params):
    params = deepcopy(params)

    if properties is None:
        properties = ['energy']

    if 'stress' in properties:
        if 'set' not in params:
            params['set'] = dict()
        params['set']['includestress'] = True

    task = params.get('task')
    if task is None:
        if 'stress' in properties or 'forces' in properties:
            task = 'gradient'
        else:
            task = 'energy'

    params = _get_kpts(atoms, **params)

    theory = _get_theory(**params)
    params['theory'] = theory
    xc = params.get('xc')
    if 'xc' in params:
        xc = _xc_conv.get(params['xc'].lower(), params['xc'])
        if theory == 'dft':
            if 'dft' not in params:
                params['dft'] = dict()
            params['dft']['xc'] = xc
        elif theory in ['pspw', 'band', 'paw']:
            if 'nwpw' not in params:
                params['nwpw'] = dict()
            params['nwpw']['xc'] = xc

    magmom_tot = int(atoms.get_initial_magnetic_moments().sum())
    params = _update_mult(magmom_tot, **params)

    label = params.get('label', 'nwchem')
    perm = os.path.abspath(params.get('perm', label))
    scratch = os.path.abspath(params.get('scratch', label))
    out = ['title "{}"'.format(label),
           'permanent_dir {}'.format(perm),
           'scratch_dir {}'.format(scratch),
           'start {}'.format(label),
           '\n'.join(_get_geom(atoms, **params)),
           '\n'.join(_get_basis(**params)),
           '\n'.join(_get_other(**params)),
           '\n'.join(_get_set(**params.get('set', dict()))),
           'task {} {}'.format(theory, task),
           '\n'.join(_get_bandpath(params.get('bandpath', None)))]

    fd.write('\n\n'.join(out))



import numpy as np
from numpy.testing import assert_allclose
from ase.calculators.lammpsrun import LAMMPS
from ase.build import bulk
from ase.test.eam_pot import Pt_u3
from ase.constraints import ExpCellFilter
from ase.optimize import BFGS

# (For now) reuse eam file stuff from other lammps test:
pot_fn = 'Pt_u3.eam'
f = open(pot_fn, 'w')
f.write(Pt_u3)
f.close()
params = {}
params['pair_style'] = 'eam'
params['pair_coeff'] = ['1 1 {}'.format(pot_fn)]
calc = LAMMPS(specorder=['Pt'], files=[pot_fn], **params)

rng = np.random.RandomState(17)

atoms = bulk('Pt') * (2, 2, 2)
atoms.rattle(stdev=0.1)
atoms.cell += 2 * rng.rand(3, 3)
atoms.calc = calc

assert_allclose(atoms.get_stress(), calc.calculate_numerical_stress(atoms),
                atol=1e-4, rtol=1e-4)

opt = BFGS(ExpCellFilter(atoms), trajectory='opt.traj')
for i, _ in enumerate(opt.irun(fmax=0.05)):
    pass

cell1_ref = np.array([
    [0.16298762, 3.89912471, 3.92825365],
    [4.21007577, 0.63362427, 5.04668170],
    [4.42895706, 3.29171414, 0.44623618]])


assert_allclose(np.asarray(atoms.cell), cell1_ref, atol=1e-4, rtol=1e-4)
assert_allclose(atoms.get_stress(), calc.calculate_numerical_stress(atoms),
                atol=1e-4, rtol=1e-4)

assert i < 80, 'Expected 59 iterations, got many more: {}'.format(i)



import sys

import numpy as np

from ase.db import connect
from ase.build import bulk
from ase.io import read, write
from ase.visualize import view
from ase.build import molecule
from ase.atoms import Atoms
from ase.symbols import string2symbols
from ase.data import ground_state_magnetic_moments
from ase.data import atomic_numbers, covalent_radii


class CLICommand:
    """Build an atom, molecule or bulk structure.

    Atom:

        ase build <chemical symbol> ...

    Molecule:

        ase build <formula> ...

    where <formula> must be one of the formulas known to ASE
    (see here: https://wiki.fysik.dtu.dk/ase/ase/build/build.html#molecules).

    Bulk:

        ase build -x <crystal structure> <formula> ...

    Examples:

        ase build Li  # lithium atom
        ase build Li -M 1  # ... with a magnetic moment of 1
        ase build Li -M 1 -V 3.5 # ... in a 7x7x7 Ang cell
        ase build H2O  # water molecule
        ase build -x fcc Cu -a 3.6  # FCC copper
    """

    @staticmethod
    def add_arguments(parser):
        add = parser.add_argument
        add('name', metavar='formula/input-file',
            help='Chemical formula or input filename.')
        add('output', nargs='?', help='Output file.')
        add('-M', '--magnetic-moment',
            metavar='M1,M2,...',
            help='Magnetic moments.  '
            'Use "-M 1" or "-M 2.3,-2.3"')
        add('--modify', metavar='...',
            help='Modify atoms with Python statement.  '
            'Example: --modify="atoms.positions[-1,2]+=0.1"')
        add('-V', '--vacuum', type=float,
            help='Amount of vacuum to add around isolated atoms '
            '(in Angstrom)')
        add('-v', '--vacuum0', type=float,
            help='Deprecated.  Use -V or --vacuum instead')
        add('--unit-cell', metavar='CELL',
            help='Unit cell in Angstrom.  Examples: "10.0" or "9,10,11"')
        add('--bond-length', type=float, metavar='LENGTH',
            help='Bond length of dimer in Angstrom')
        add('-x', '--crystal-structure',
            help='Crystal structure',
            choices=['sc', 'fcc', 'bcc', 'hcp', 'diamond',
                     'zincblende', 'rocksalt', 'cesiumchloride',
                     'fluorite', 'wurtzite'])
        add('-a', '--lattice-constant', default='', metavar='LENGTH',
            help='Lattice constant or comma-separated lattice constantes in '
            'Angstrom')
        add('--orthorhombic', action='store_true',
            help='Use orthorhombic unit cell')
        add('--cubic', action='store_true',
            help='Use cubic unit cell')
        add('-r', '--repeat',
            help='Repeat unit cell.  Use "-r 2" or "-r 2,3,1"')
        add('-g', '--gui', action='store_true',
            help='open ase gui')
        add('--periodic', action='store_true',
            help='make structure fully periodic')

    @staticmethod
    def run(args, parser):
        if args.vacuum0:
            parser.error('Please use -V or --vacuum instead!')

        if '.' in args.name:
            # Read from file:
            atoms = read(args.name)
        elif args.crystal_structure:
            atoms = build_bulk(args)
        else:
            atoms = build_molecule(args)

        if args.magnetic_moment:
            magmoms = np.array(
                [float(m) for m in args.magnetic_moment.split(',')])
            atoms.set_initial_magnetic_moments(
                np.tile(magmoms, len(atoms) // len(magmoms)))

        if args.modify:
            exec(args.modify, {'atoms': atoms})

        if args.repeat is not None:
            r = args.repeat.split(',')
            if len(r) == 1:
                r = 3 * r
            atoms = atoms.repeat([int(c) for c in r])

        if args.gui:
            view(atoms)

        if args.output:
            write(args.output, atoms)
        elif sys.stdout.isatty():
            write(args.name + '.json', atoms)
        else:
            con = connect(sys.stdout, type='json')
            con.write(atoms, name=args.name)


def build_molecule(args):
    try:
        # Known molecule or atom?
        atoms = molecule(args.name)
    except NotImplementedError:
        symbols = string2symbols(args.name)
        if len(symbols) == 1:
            Z = atomic_numbers[symbols[0]]
            magmom = ground_state_magnetic_moments[Z]
            atoms = Atoms(args.name, magmoms=[magmom])
        elif len(symbols) == 2:
            # Dimer
            if args.bond_length is None:
                b = (covalent_radii[atomic_numbers[symbols[0]]] +
                     covalent_radii[atomic_numbers[symbols[1]]])
            else:
                b = args.bond_length
            atoms = Atoms(args.name, positions=[(0, 0, 0),
                                                (b, 0, 0)])
        else:
            raise ValueError('Unknown molecule: ' + args.name)
    else:
        if len(atoms) == 2 and args.bond_length is not None:
            atoms.set_distance(0, 1, args.bond_length)

    if args.unit_cell is None:
        if args.vacuum:
            atoms.center(vacuum=args.vacuum)
        else:
            atoms.center(about=[0, 0, 0])
    else:
        a = [float(x) for x in args.unit_cell.split(',')]
        if len(a) == 1:
            cell = [a[0], a[0], a[0]]
        elif len(a) == 3:
            cell = a
        else:
            a, b, c, alpha, beta, gamma = a
            degree = np.pi / 180.0
            cosa = np.cos(alpha * degree)
            cosb = np.cos(beta * degree)
            sinb = np.sin(beta * degree)
            cosg = np.cos(gamma * degree)
            sing = np.sin(gamma * degree)
            cell = [[a, 0, 0],
                    [b * cosg, b * sing, 0],
                    [c * cosb, c * (cosa - cosb * cosg) / sing,
                     c * np.sqrt(
                        sinb**2 - ((cosa - cosb * cosg) / sing)**2)]]
        atoms.cell = cell
        atoms.center()

    atoms.pbc = args.periodic

    return atoms


def build_bulk(args):
    L = args.lattice_constant.replace(',', ' ').split()
    d = dict([(key, float(x)) for key, x in zip('ac', L)])
    atoms = bulk(args.name, crystalstructure=args.crystal_structure,
                 a=d.get('a'), c=d.get('c'),
                 orthorhombic=args.orthorhombic, cubic=args.cubic)

    M, X = {'Fe': (2.3, 'bcc'),
            'Co': (1.2, 'hcp'),
            'Ni': (0.6, 'fcc')}.get(args.name, (None, None))
    if M is not None and args.crystal_structure == X:
        atoms.set_initial_magnetic_moments([M] * len(atoms))

    return atoms



"""
Check that the coordinates registered with the KIM API are updated
appropriately when the atomic positions are updated.  This can go awry
if the 'coords' attribute of the relevant NeighborList subclass is
reassigned to a new memory location -- a problem which was indeed
occurring at one point (see https://gitlab.com/ase/ase/merge_requests/1442)!
"""
import numpy as np
from ase import Atoms
from ase.calculators.kim import KIM


def squeeze_dimer(atoms, d):
    """Squeeze the atoms together by the absolute distance ``d`` (Angstroms)
    """
    pos = atoms.get_positions()
    pos[0] += np.asarray([d, 0, 0])
    atoms.set_positions(pos)


def set_positions_to_orig(atoms, box_len, dimer_separation):
    pos1 = np.asarray([box_len / 2.0, box_len / 2.0, box_len / 2.0]) - np.asarray(
        [dimer_separation / 2.0, 0, 0]
    )
    pos2 = np.asarray([box_len / 2.0, box_len / 2.0, box_len / 2.0]) + np.asarray(
        [dimer_separation / 2.0, 0, 0]
    )
    atoms.set_positions([pos1, pos2])


# We know that ex_model_Ar_P_Morse_07C has a cutoff of 8.15 Angstroms
model = "ex_model_Ar_P_Morse_07C"
model_cutoff = 8.15  # Angstroms

# Create a dimer centered in a small box that's small enough so that there will be
# padding atoms when we make it periodic
box_len = 0.5 * model_cutoff
dimer_separation = model_cutoff * 0.3

atoms = Atoms("Ar" * 2, cell=[[box_len, 0, 0], [0, box_len, 0], [0, 0, box_len]])

# Create calculator.  Either the kimpy neighbor list library or ASE's native neighbor
# lists should suffice to check this since update_kim_coords() belongs to their parent
# class, NeighborList.  Here, we'll use the default mode (kimpy neighbor list).
neigh_skin_ratio = 0.2
skin = neigh_skin_ratio * model_cutoff
calc = KIM(model, options={"neigh_skin_ratio": neigh_skin_ratio})
atoms.set_calculator(calc)

squeezed_energies_ref = {
    False: 5.784620078721877,  # finite
    True: 6.766293119162073,  # periodic
}

for pbc in [False, True]:

    # Reset dimer positions to original configuration
    set_positions_to_orig(atoms, box_len, dimer_separation)

    atoms.set_pbc(pbc)

    # Get potential energy so that it will get rid of "pbc" being in the system_changes.
    # The only way that update_kim_coords is called is when system_changes
    # only contains "positions", as otherwise a neighbor list rebuild is triggered.
    atoms.get_potential_energy()

    # First squeeze the dimer together by a distance less than the skin and compute the
    # energy.  This doesn't trigger a neighbor list rebuild (which we avoid here because
    # we're only trying to test update_kim_coords, which is not called in the event that
    # a neighbor list rebuild is necessary).
    #
    # This will update the coordinate values in the ``coords`` attribute of the relevant
    # NeighborList subclass instance (see ase/calculators/kim/neighborlist.py) and,
    # since ``coords`` is pointing at the same memory the KIM API is reading the
    # coordinates from, the KIM Model will see the updated coordinates with no problem.
    # *However*, it's possible that after updating these values, the ``coords``
    # attribute is bound to a *new* object in memory if update_kim_coords is broken
    # somehow!
    squeeze_dimer(atoms, 0.2 * skin)
    atoms.get_potential_energy()

    # Now squeeze the dimer together by the same amount again, where the sum of this
    # squeeze and the last is still less than the skin distance so that we still avoid a
    # neighbor list rebuild.  If all is well, the `coords` attribute of the relevant
    # NeighborList subclass still points at the same location as before we did the
    # previous squeeze.  Thus, when we update the coordinates again, it should still be
    # updating the same coordinates being read by the KIM API, giving us the expected
    # value of the energy.  If update_kim_coords is broken, ``coords`` will already be
    # bound to a new object in memory while the KIM API is still looking at its previous
    # address to read coordinates.  This means that when we update the coordinates in
    # the ``coords`` value, the KIM API never sees them.  In fact, it's not even
    # guaranteed that the KIM API will read the same coordinates as before since the
    # memory where it's looking may have since been overwritten by some other python
    # objects.
    squeeze_dimer(atoms, 0.2 * skin)
    squeezed_energy = atoms.get_potential_energy()
    assert np.isclose(squeezed_energy, squeezed_energies_ref[pbc])



from ase import Atoms
from ase.io import read, write
from ase.parallel import world

n = world.rank + 1
a = Atoms('H' * n)
name = 'H{}.xyz'.format(n)
write(name, a, parallel=False)
b = read(name, parallel=False)
assert n == len(b)



"""Water dimer calculation in which each molecule is calculated quantum
mechanically and the interaction between the molecules is electrostatic.
The process is repeated until self consitence. """

from numpy.linalg import norm
from ase.collections import s22
from ase.calculators.turbomole import Turbomole


def polarization_cycle(partition_1, partition_2, charges_2=None):
    """Performs an iteration of a polarization calculation."""
    properties = {}
    calc = Turbomole(atoms=partition_1, **params)
    if charges_2 is not None:
        calc.embed(charges=charges_2, positions=partition_2.positions)
    properties['e1'] = partition_1.get_potential_energy()
    properties['c1'] = partition_1.get_charges()
    calc = Turbomole(atoms=partition_2, **params)
    calc.embed(charges=properties['c1'], positions=partition_1.positions)
    properties['e2'] = partition_2.get_potential_energy()
    properties['c2'] = partition_2.get_charges()
    return properties


params = {'esp fit': 'kollman', 'multiplicity': 1}
dimer = s22['Water_dimer']

# system partitioning
part1 = dimer[0:3]
part2 = dimer[3:6]

new = polarization_cycle(part1, part2)
prop = {'e1': [], 'e2': [], 'c1': [], 'c2': []}
for key in prop:
    prop[key].append(new[key])

# start values and convergence criteria
conv = {'e': [1.0], 'c': [1.0]}
thres = {'e': 1e-4, 'c': 1e-2}
iteration = 0
while any([conv[key][-1] > thres[key] for key in conv]):
    iteration += 1
    new = polarization_cycle(part1, part2, charges_2=prop['c2'][-1])
    for key in prop:
        prop[key].append(new[key])

    (new1, old1) = (prop['e1'][-1], prop['e1'][-2])
    (new2, old2) = (prop['e2'][-1], prop['e2'][-2])
    conv['e'].append((abs(new1 - old1) + abs(new2 - old2)) /
                     (abs(old1) + abs(old2)))
    (new1, old1) = (prop['c1'][-1], prop['c1'][-2])
    (new2, old2) = (prop['c2'][-1], prop['c2'][-2])
    conv['c'].append((norm(new1 - old1) + norm(new2 - old2)) /
                     (norm(old1) + norm(old2)))
    fmt = 'iteration {0:d}: convergence of energy {1:10e}; of charges {2:10e}'
    print(fmt.format(iteration, conv['e'][-1], norm(conv['c'][-1])))

# check the result
ref = {
    'e1': -2077.7082947500003,
    'e2': -2077.3347674372353,
    'c1': [-0.133033, 0.238218, -0.105186],
    'c2': [-0.844336, 0.422151, 0.422184]
}

dev = {}
for key in ref:
    val = prop[key][-1]
    measurement = val if isinstance(val, float) else norm(val)
    reference = ref[key] if isinstance(ref[key], float) else norm(ref[key])
    dev[key] = (measurement - reference) / reference
    print('Deviation of {0} is {1:10f}'.format(key, dev[key]))

# allow deviations of up to 5%
assert all([dev[key] < 5e-2 for key in dev]), 'deviation too large'



from ase.build import bulk
from ase.calculators.nwchem import NWChem
from ase.utils import workdir
from ase.dft.band_structure import calculate_band_structure

atoms = bulk('Si')
with workdir('files', mkdir=True):
    path = atoms.cell.bandpath('GXWK', density=10)
    atoms.calc = NWChem(kpts=[2, 2, 2])
    bs = calculate_band_structure(atoms, path)
    print(bs)
    bs.write('bs.json')



import numpy as np
from ase.calculators.calculator import FileIOCalculator
from ase.calculators.calculator import SCFError
import ase.units


class QChem(FileIOCalculator):
    """
    QChem calculator
    """
    name = 'QChem'

    implemented_properties = ['energy', 'forces']
    command = 'qchem PREFIX.inp PREFIX.out'

    # Following the minimal requirements given in
    # http://www.q-chem.com/qchem-website/manual/qchem43_manual/sect-METHOD.html
    default_parameters = {'method': 'hf',
                          'basis': '6-31G*',
                          'jobtype': None,
                          'charge': 0}

    def __init__(self, restart=None, ignore_bad_restart_file=False,
                 label='qchem', scratch=None, np=1, nt=1, pbs=False,
                 basisfile=None, ecpfile=None, atoms=None, **kwargs):
        """
        The scratch directory, number of processor and threads as well as a few
        other command line options can be set using the arguments explained
        below. The remaining kwargs are copied as options to the input file.
        The calculator will convert these options to lower case for storage
        and convert back to upper case (Q-Chem standard) when writting the
        input file.

        scratch: str
            path of the scratch directory
        np: int
            number of processors for the -np command line flag
        nt: int
            number of threads for the -nt command line flag
        pbs: boolean
            command line flag for pbs scheduler (see Q-Chem manual)
        basisfile: str
            path to file containing the basis. Use in combination with
            basis='gen' keyword argument.
        ecpfile: str
            path to file containing the effective core potential. Use in
            combination with ecp='gen' keyword argument.
        """

        FileIOCalculator.__init__(self, restart, ignore_bad_restart_file,
                                  label, atoms, **kwargs)

        # Augment the command by various flags
        if pbs:
            self.command = 'qchem -pbs '
        else:
            self.command = 'qchem '
        if np != 1:
            self.command += '-np %d ' % np
        if nt != 1:
            self.command += '-nt %d ' % nt
        self.command += 'PREFIX.inp PREFIX.out'
        if scratch is not None:
            self.command += ' %s' % scratch

        self.basisfile = basisfile
        self.ecpfile = ecpfile

    def read(self, label):
        raise NotImplementedError

    def read_results(self):
        filename = self.label + '.out'

        with open(filename, 'r') as fileobj:
            lineiter = iter(fileobj)
            for line in lineiter:
                if 'SCF failed to converge' in line:
                    raise SCFError()
                elif 'ERROR: alpha_min' in line:
                    # Even though it is not technically a SCFError:
                    raise SCFError()
                elif ' Total energy in the final basis set =' in line:
                    convert = ase.units.Hartree
                    self.results['energy'] = float(line.split()[8]) * convert
                elif ' Gradient of SCF Energy' in line:
                    # Read gradient as 3 by N array and transpose at the end
                    gradient = [[] for _ in range(3)]
                    # Skip first line containing atom numbering
                    next(lineiter)
                    while True:
                        # Loop over the three Cartesian coordinates
                        for i in range(3):
                            # Cut off the component numbering and remove
                            # trailing characters ('\n' and stuff)
                            line = next(lineiter)[5:].rstrip()
                            # Cut in chunks of 12 symbols and convert into
                            # strings. This is prefered over string.split() as
                            # the fields may overlap for large gradients
                            gradient[i].extend(list(map(
                                float, [line[i:i + 12]
                                        for i in range(0, len(line), 12)])))

                        # After three force components we expect either a
                        # separator line, which we want to skip, or the end of
                        # the gradient matrix which is characterized by the
                        # line ' Max gradient component'.
                        # Maybe change stopping criterion to be independent of
                        # next line. Eg. if not lineiter.next().startswith(' ')
                        if ' Max gradient component' in next(lineiter):
                            # Minus to convert from gradient to force
                            self.results['forces'] = np.array(gradient).T * (
                                -ase.units.Hartree / ase.units.Bohr)
                            break

    def write_input(self, atoms, properties=None, system_changes=None):
        FileIOCalculator.write_input(self, atoms, properties, system_changes)
        filename = self.label + '.inp'

        with open(filename, 'w') as fileobj:
            fileobj.write('$comment\n   ASE generated input file\n$end\n\n')

            fileobj.write('$rem\n')
            if self.parameters['jobtype'] is None:
                if 'forces' in properties:
                    fileobj.write('   %-25s   %s\n' % ('JOBTYPE', 'FORCE'))
                else:
                    fileobj.write('   %-25s   %s\n' % ('JOBTYPE', 'SP'))

            for prm in self.parameters:
                if prm not in ['charge', 'multiplicity']:
                    if self.parameters[prm] is not None:
                        fileobj.write('   %-25s   %s\n' % (
                            prm.upper(), self.parameters[prm].upper()))

            # Not even a parameters as this is an absolute necessity
            fileobj.write('   %-25s   %s\n' % ('SYM_IGNORE', 'TRUE'))
            fileobj.write('$end\n\n')

            fileobj.write('$molecule\n')
            # Following the example set by the gaussian calculator
            if ('multiplicity' not in self.parameters):
                tot_magmom = atoms.get_initial_magnetic_moments().sum()
                mult = tot_magmom + 1
            else:
                mult = self.parameters['multiplicity']
            # Default charge of 0 is defined in default_parameters
            fileobj.write('   %d %d\n' % (self.parameters['charge'], mult))
            for a in atoms:
                fileobj.write('   %s  %f  %f  %f\n' % (a.symbol,
                                                       a.x, a.y, a.z))
            fileobj.write('$end\n\n')

            if self.basisfile is not None:
                with open(self.basisfile, 'r') as f_in:
                    basis = f_in.readlines()
                fileobj.write('$basis\n')
                fileobj.writelines(basis)
                fileobj.write('$end\n\n')

            if self.ecpfile is not None:
                with open(self.ecpfile, 'r') as f_in:
                    ecp = f_in.readlines()
                fileobj.write('$ecp\n')
                fileobj.writelines(ecp)
                fileobj.write('$end\n\n')



from ase.io.formats import ioformats

traj = ioformats['traj']
print(traj)

outcar = ioformats['vasp-out']
print(outcar)
assert outcar.match_name('OUTCAR')
assert outcar.match_name('something.with.OUTCAR.stuff')



from ase import Atoms
from ase.calculators.emt import EMT
from ase.constraints import FixAtoms
from ase.optimize import QuasiNewton
from ase.io import write

# Find the initial and final states for the reaction.

# Set up a (3 x 3) two layer slab of Ru:
a = 2.70
c = 1.59 * a
sqrt3 = 3. ** .5
bulk = Atoms('2Cu', [(0., 0., 0.), (1./3, 1./3, -0.5*c)],
             tags=(1, 1),
             pbc=(1, 1, 0))
bulk.set_cell([(a,     0,              0),
               (a / 2, sqrt3 * a / 2, 0),
               (0,     0,              1)])
slab = bulk.repeat((4, 4, 1))

# Initial state.
# Add the molecule:
x = a / 2.
y = a * 3. ** .5 / 6.
z = 1.8
d = 1.10 # N2 bond length

# Molecular state parallel to the surface:
slab += Atoms('2N', [(x, y, z), (x + sqrt3 * d / 2, y + d / 2, z)])

# Use the EMT calculator for the forces and energies:
slab.set_calculator(EMT())

# We don't want to worry about the Cu degrees of freedom:
mask = [atom.symbol == 'Cu' for atom in slab]
slab.set_constraint(FixAtoms(mask=mask))
relax = QuasiNewton(slab)
relax.run(fmax=0.05)
print('initial state:', slab.get_potential_energy())
write('N2.traj', slab)

# Now the final state.
# Move the second N atom to a neighboring hollow site:
slab[-1].position = (x + a, y, z)
relax.run()
print('final state:  ', slab.get_potential_energy())
write('2N.traj', slab)



# -*- coding: utf-8 -*-

"""Test suit for the CP2K ASE calulator.

http://www.cp2k.org
Author: Ole Schuett <ole.schuett@mat.ethz.ch>
"""

from ase.build import molecule
from ase.calculators.cp2k import CP2K


def main():
    calc = CP2K(xc='XC_GGA_X_PBE XC_GGA_C_PBE',
                pseudo_potential="GTH-PBE",
                label='test_H2_libxc')
    h2 = molecule('H2', calculator=calc)
    h2.center(vacuum=2.0)
    energy = h2.get_potential_energy()
    energy_ref = -31.591716529642
    diff = abs((energy - energy_ref) / energy_ref)
    assert diff < 1e-10
    print('passed test "H2_libxc"')


main()



import numpy as np
from ase import Atoms
from ase.constraints import FixAtoms
from ase.calculators.emt import EMT
from ase.neb import NEB
from ase.visualize import view
from ase.optimize.fire import FIRE as QuasiNewton
from ase.lattice.cubic import FaceCenteredCubic

#set the number of images you want
nimages = 5

#some algebra to determine surface normal and the plane of the surface
d3=[2,1,1]
a1=np.array([0,1,1])
d1=np.cross(a1,d3)
a2=np.array([0,-1,1])
d2=np.cross(a2,d3)

#create your slab
slab  =FaceCenteredCubic(directions=[d1,d2,d3],
                         size=(2,1,2),
                         symbol=('Pt'),
                         latticeconstant=3.9)

#add some vacuum to your slab
uc = slab.get_cell()
print(uc)
uc[2] += [0,0,10]  #there are ten layers of vacuum
uc = slab.set_cell(uc,scale_atoms=False)
#view the slab to make sure it is how you expect
view(slab)

#some positions needed to place the atom in the correct place
x1 = 1.379
x2 = 4.137
x3 = 2.759
y1 = 0.0
y2 = 2.238
z1 = 7.165
z2 = 6.439


#Add the adatom to the list of atoms and set constraints of surface atoms.
slab += Atoms('N', [ ((x2+x1)/2,y1,z1+1.5)])
mask = [atom.symbol == 'Pt' for atom in slab]
slab.set_constraint(FixAtoms(mask=mask))

#optimise the initial state
# Atom below step
initial = slab.copy()
initial.set_calculator(EMT())
relax = QuasiNewton(initial)
relax.run(fmax=0.05)
view(initial)

#optimise the initial state
# Atom above step
slab[-1].position = (x3,y2+1,z2+3.5)
final = slab.copy()
final.set_calculator(EMT())
relax = QuasiNewton(final)
relax.run(fmax=0.05)
view(final)

#create a list of images for interpolation
images = [initial]
for i in range(nimages):
    images.append(initial.copy())

for image in images:
    image.set_calculator(EMT())

images.append(final)
view(images)

#carry out idpp interpolation
neb = NEB(images)
neb.interpolate('idpp')

#Run NEB calculation
qn = QuasiNewton(neb, trajectory='N_diffusion.traj',  logfile='N_diffusion.log')
qn.run(fmax=0.05)





from ase.gui.i18n import _

import ase.data
import ase.gui.ui as ui

from ase import Atoms
from ase.collections import g2


class Element(list):
    def __init__(self, symbol='', callback=None, allow_molecule=False):
        list.__init__(self,
                      [_('Element:'),
                       ui.Entry(symbol, 10 if allow_molecule else 3,
                                self.enter),
                       ui.Button(_('Help'), self.show_help),
                       ui.Label('', 'red')])
        self.callback = callback
        self.allow_molecule = allow_molecule

    def grab_focus(self):
        self[1].entry.focus_set()

    def show_help(self):
        names = []
        import re
        for name in g2.names:
            if not re.match('^[A-Z][a-z]?$', name):  # Not single atoms
                names.append(name)

        # This infobox is indescribably ugly because of the
        # ridiculously large font size used by Tkinter.  Ouch!
        msg = _('Enter a chemical symbol or the name of a molecule '
                'from the G2 testset:\n'
                '{}'.format(', '.join(names)))
        ui.showinfo('Info', msg)

    @property
    def Z(self):
        assert not self.allow_molecule
        atoms = self.get_atoms()
        if atoms is None:
            return None
        assert len(atoms) == 1
        return atoms.numbers[0]

    @property
    def symbol(self):
        Z = self.Z
        return None if Z is None else ase.data.chemical_symbols[Z]

    # Used by tests...
    @symbol.setter
    def symbol(self, value):
        self[1].value = value

    def get_atoms(self):
        val = self._get()
        if val is not None:
            self[2].text = ''
        return val

    def _get(self):
        txt = self[1].value

        if not txt:
            self.error(_('No element specified!'))
            return None

        if txt.isdigit():
            txt = int(txt)
            try:
                txt = ase.data.chemical_symbols[txt]
            except KeyError:
                self.error()
                return None

        if txt in ase.data.atomic_numbers:
            return Atoms(txt)

        if self.allow_molecule and g2.has(txt):
            return g2[txt]

        self.error()

    def enter(self):
        self.callback(self)

    def error(self, text=_('ERROR: Invalid element!')):
        self[2].text = text


def pybutton(title, callback):
    """A button for displaying Python code.

    When pressed, it opens a window displaying some Python code, or an error
    message if no Python code is ready.
    """
    return ui.Button('Python', pywindow, title, callback)


def pywindow(title, callback):
    code = callback()
    if code is None:
        ui.error(
            _('No Python code'),
            _('You have not (yet) specified a consistent set of parameters.'))
    else:
        win = ui.Window(title)
        win.add(ui.Text(code))



from ase.calculators.calculator import get_calculator_class
from ase.io import read, write
from ase.build import molecule
from ase.test import test_calculator_names
from ase.utils import workdir


def h2(name, par):
    h2 = molecule('H2', pbc=par.pop('pbc', False))
    h2.center(vacuum=2.0)
    h2.calc = get_calculator_class(name)(**par)
    e = h2.get_potential_energy()
    assert not h2.calc.calculation_required(h2, ['energy'])
    f = h2.get_forces()
    assert not h2.calc.calculation_required(h2, ['energy', 'forces'])
    write('h2.traj', h2)
    h2 = read('h2.traj')
    assert abs(e - h2.get_potential_energy()) < 1e-12
    assert abs(f - h2.get_forces()).max() < 1e-12



def run(name, par):
    if name not in test_calculator_names:
        return
    with workdir(name + '-test', mkdir=True):
        h2(name, par)



run('abinit', dict(ecut=200, toldfe=0.0001))
run('aims', dict(sc_accuracy_rho=5.e-3, sc_accuracy_forces=1e-4, xc='LDA'))
run('crystal', dict(basis='sto-3g'))
run('gpaw', dict(mode='lcao',  basis='sz(dzp)'))
run('elk', dict(tasks=0, rgkmax=5.0, epsengy=1.0, epspot=1.0, tforce=True,
                pbc=True))
run('jacapo', dict(pbc=True))
run('vasp', dict(xc='LDA'))
run('Psi4', {})
# XXX we don't have the pseudopotential for Espresso.
# We need some kind of installation/config system for managing that.
# Disabling espresso test until we get that. --askhl
#'espresso': dict(pbc=True, tprnfor=True,
#                 pseudopotentials={'H': 'H.pbe-rrkjus_psl.0.1.UPF'}),
run('emt', {})



from ase.calculators.gaussian import Gaussian
from ase.atoms import Atoms
from ase.optimize.lbfgs import LBFGS


# First test to make sure Gaussian works
calc = Gaussian(method='pbepbe', basis='sto-3g', force='force',
                nproc=1, chk='water.chk', label='water')
calc.clean()

water = Atoms('OHH',
              positions=[(0, 0, 0), (1, 0, 0), (0, 1, 0)],
              calculator=calc)

opt = LBFGS(water)
opt.run(fmax=0.05)

forces = water.get_forces()
energy = water.get_potential_energy()
positions = water.get_positions()

# Then test the IO routines
from ase.io import read
water2 = read('water.log')
forces2 = water2.get_forces()
energy2 = water2.get_potential_energy()
positions2 = water2.get_positions()
#compare distances since positions are different in standard orientation
dist = water.get_all_distances()
dist2 = read('water.log', quantity='structures')[-1].get_all_distances()

assert abs(energy - energy2) < 1e-7
assert abs(forces - forces2).max() < 1e-9
assert abs(positions - positions2).max() < 1e-6
assert abs(dist - dist2).max() < 1e-6



from math import cos, sin, pi

import numpy as np
#import matplotlib.pyplot as plt

import ase.units as units
from ase import Atoms
from ase.calculators.tip4p import TIP4P, epsilon0, sigma0, rOH, angleHOH
from ase.calculators.qmmm import (SimpleQMMM, EIQMMM, LJInteractions,
                                  LJInteractionsGeneral)
from ase.constraints import FixInternals
from ase.optimize import BFGS

r = rOH
a = angleHOH * pi / 180

# From http://dx.doi.org/10.1063/1.445869
eexp = 6.24 * units.kcal / units.mol
dexp = 2.75
aexp = 46

D = np.linspace(2.5, 3.5, 30)

i = LJInteractions({('O', 'O'): (epsilon0, sigma0)})

# General LJ interaction object
sigma_mm = np.array([sigma0, 0, 0])
epsilon_mm = np.array([epsilon0, 0, 0])
sigma_qm = np.array([sigma0, 0, 0])
epsilon_qm = np.array([epsilon0, 0, 0])
ig = LJInteractionsGeneral(sigma_qm, epsilon_qm, sigma_mm, epsilon_mm, 3)

for calc in [TIP4P(),
             SimpleQMMM([0, 1, 2], TIP4P(), TIP4P(), TIP4P()),
             SimpleQMMM([0, 1, 2], TIP4P(), TIP4P(), TIP4P(), vacuum=3.0),
             EIQMMM([0, 1, 2], TIP4P(), TIP4P(), i),
             EIQMMM([3, 4, 5], TIP4P(), TIP4P(), i, vacuum=3.0),
             EIQMMM([0, 1, 2], TIP4P(), TIP4P(), i, vacuum=3.0),
             EIQMMM([0, 1, 2], TIP4P(), TIP4P(), ig),
             EIQMMM([3, 4, 5], TIP4P(), TIP4P(), ig, vacuum=3.0),
             EIQMMM([0, 1, 2], TIP4P(), TIP4P(), ig, vacuum=3.0)]:
    dimer = Atoms('OH2OH2',
                  [(0, 0, 0),
                   (r * cos(a), 0, r * sin(a)),
                   (r, 0, 0),
                   (0, 0, 0),
                   (r * cos(a / 2), r * sin(a / 2), 0),
                   (r * cos(a / 2), -r * sin(a / 2), 0)
                   ])
    dimer.calc = calc

    E = []
    F = []
    for d in D:
        dimer.positions[3:, 0] += d - dimer.positions[3, 0]
        E.append(dimer.get_potential_energy())
        F.append(dimer.get_forces())

    F = np.array(F)

    #plt.plot(D, E)

    F1 = np.polyval(np.polyder(np.polyfit(D, E, 7)), D)
    F2 = F[:, :3, 0].sum(1)
    error = abs(F1 - F2).max()

    dimer.constraints = FixInternals(
        bonds=[(r, (0, 1)), (r, (0, 2)),
               (r, (3, 4)), (r, (3, 5))],
        angles=[(a, (2, 0, 1)), (a, (5, 3, 4))])
    opt = BFGS(dimer,
               trajectory=calc.name + '.traj', logfile=calc.name + 'd.log')
    opt.run(0.01)

    e0 = dimer.get_potential_energy()
    d0 = dimer.get_distance(0, 3)
    R = dimer.positions
    v1 = R[2] - R[3]
    v2 = R[3] - (R[5] + R[4]) / 2
    a0 = np.arccos(np.dot(v1, v2) /
                   (np.dot(v1, v1) * np.dot(v2, v2))**0.5) / np.pi * 180
    fmt = '{0:>23}: {1:.3f} {2:.3f} {3:.3f} {4:.1f}'
    print(fmt.format(calc.name, -min(E), -e0, d0, a0))
    assert abs(e0 + eexp) < 0.002
    assert abs(d0 - dexp) < 0.006
    assert abs(a0 - aexp) < 2.5

print(fmt.format('reference', 9.999, eexp, dexp, aexp))

#plt.show()



"Base class for simulation windows"

from ase.gui.i18n import _

import ase.gui.ui as ui
#from ase import Atoms
#from ase.constraints import FixAtoms

raise NotImplementedError('Module not ported to tkinter')

pack = error = 42


class Simulation:
    def __init__(self, gui):
        ui.Window.__init__(self)
        self.gui = gui

    def packtext(self, vbox, text, label=None):
        "Pack an text frame into the window."
        pack(vbox, ui.Label(""))
        txtframe = ui.Frame(label)
        txtlbl = ui.Label(text)
        txtframe.add(txtlbl)
        txtlbl.show()
        pack(vbox, txtframe)
        pack(vbox, ui.Label(""))

    def packimageselection(self,
                           outerbox,
                           txt1=_(" (rerun simulation)"),
                           txt2=_(" (continue simulation)")):
        "Make the frame for selecting starting config if more than one."
        self.startframe = ui.Frame(_("Select starting configuration:"))
        pack(outerbox, [self.startframe])
        vbox = ui.VBox()
        self.startframe.add(vbox)
        vbox.show()
        self.numconfig_format = _("There are currently %i "
                                  "configurations loaded.")
        self.numconfig_label = ui.Label("")
        pack(vbox, [self.numconfig_label])
        lbl = ui.Label(
            _("Choose which one to use as the "
              "initial configuration"))
        pack(vbox, [lbl])
        self.start_radio_first = ui.RadioButton(
            None, _("The first configuration %s.") % txt1)
        pack(vbox, [self.start_radio_first])
        self.start_radio_nth = ui.RadioButton(self.start_radio_first,
                                              _("Configuration number "))
        self.start_nth_adj = ui.Adjustment(0, 0, 1, 1)
        self.start_nth_spin = ui.SpinButton(self.start_nth_adj, 0, 0)
        self.start_nth_spin.set_sensitive(False)
        pack(vbox, [self.start_radio_nth, self.start_nth_spin])
        self.start_radio_last = ui.RadioButton(
            self.start_radio_first, _("The last configuration %s.") % txt2)
        self.start_radio_last.set_active(True)
        pack(vbox, self.start_radio_last)
        self.start_radio_nth.connect("toggled", self.start_radio_nth_toggled)
        self.setupimageselection()

    def start_radio_nth_toggled(self, widget):
        self.start_nth_spin.set_sensitive(self.start_radio_nth.get_active())

    def setupimageselection(self):
        "Decide if the start image selection frame should be shown."
        n = len(self.gui.images)
        if n <= 1:
            self.startframe.hide()
        else:
            self.startframe.show()
            if self.start_nth_adj.value >= n:
                self.start_nth_adj.value = n - 1
            self.start_nth_adj.upper = n - 1
            self.numconfig_label.set_text(self.numconfig_format % (n, ))

    def getimagenumber(self):
        "Get the image number selected in the start image frame."
        nmax = len(self.gui.images)
        if nmax <= 1:
            return 0
        elif self.start_radio_first.get_active():
            return 0
        elif self.start_radio_nth.get_active():
            return self.start_nth_adj.value
        else:
            assert self.start_radio_last.get_active()
            return nmax - 1

    def makebutbox(self, vbox, helptext=None):
        self.buttons = ui.HButtonBox()
        runbut = ui.Button(_("Run"))
        runbut.connect('clicked', self.run)
        closebut = ui.Button('Close')
        closebut.connect('clicked', lambda x: self.destroy())
        for w in (runbut, closebut):
            self.buttons.pack_start(w, 0, 0)
            w.show()
        if helptext:
            helpbut = [help(helptext)]
        else:
            helpbut = []
        pack(vbox, helpbut + [self.buttons], end=True, bottom=True)

    def setup_atoms(self):
        self.atoms = self.get_atoms()
        if self.atoms is None:
            return False
        try:
            self.calculator = self.gui.simulation['calc']
        except KeyError:
            error(_("No calculator: Use Calculate/Set Calculator on the menu."))
            return False
        self.atoms.set_calculator(self.calculator())
        return True

    def get_atoms(self):
        "Make an atoms object from the active image"
        images = self.gui.images
        atoms = images[self.getimagenumber()]
        natoms = len(atoms) // images.repeat.prod()
        if natoms < 1:
            error(_("No atoms present"))
            return None
        return atoms[:natoms]

    def begin(self, **kwargs):
        if 'progress' in self.gui.simulation:
            self.gui.simulation['progress'].begin(**kwargs)

    def end(self):
        if 'progress' in self.gui.simulation:
            self.gui.simulation['progress'].end()

    def prepare_store_atoms(self):
        "Informs the gui that the next configuration should be the first."
        self.gui.prepare_new_atoms()
        self.count_steps = 0

    def store_atoms(self):
        "Observes the minimization and stores the atoms in the gui."
        self.gui.append_atoms(self.atoms)
        self.count_steps += 1



from math import sqrt
from ase import Atom, Atoms
from ase.neb import NEB
from ase.constraints import FixAtoms
from ase.vibrations import Vibrations
from ase.visualize import view
from ase.calculators.emt import EMT
from ase.optimize import QuasiNewton, BFGS

# Distance between Cu atoms on a (100) surface:
d = 3.6 / sqrt(2)
initial = Atoms('Cu',
                positions=[(0, 0, 0)],
                cell=(d, d, 1.0),
                pbc=(True, True, False))
initial *= (2, 2, 1)  # 2x2 (100) surface-cell

# Approximate height of Ag atom on Cu(100) surfece:
h0 = 2.0
initial += Atom('Ag', (d / 2, d / 2, h0))

if 0:
    view(initial)

# Make band:
images = [initial.copy() for i in range(6)]
neb = NEB(images, climb=True)

# Set constraints and calculator:
constraint = FixAtoms(range(len(initial) - 1))
for image in images:
    image.set_calculator(EMT())
    image.set_constraint(constraint)

# Displace last image:
images[-1].positions[-1] += (d, 0, 0)
#images[-1].positions[-1] += (d, d, 0)

# Relax height of Ag atom for initial and final states:
dyn1 = QuasiNewton(images[0])
dyn1.run(fmax=0.01)
dyn2 = QuasiNewton(images[-1])
dyn2.run(fmax=0.01)

# Interpolate positions between initial and final states:
neb.interpolate()

for image in images:
    print(image.positions[-1], image.get_potential_energy())

#dyn = MDMin(neb, dt=0.4)
#dyn = FIRE(neb, dt=0.4)
dyn = BFGS(neb, trajectory='mep.traj')
dyn.run(fmax=0.05)

for image in images:
    print(image.positions[-1], image.get_potential_energy())

a = images[0]
vib = Vibrations(a, [4])
vib.run()
print(vib.get_frequencies())
vib.summary()
print(vib.get_mode(-1))
vib.write_mode(-1, nimages=20)



import numpy as np

from ase.atoms import Atoms
from ase.utils import basestring


def write_plt(filename, atoms, data):
    if isinstance(atoms, Atoms):
        cell = atoms.get_cell()
    else:
        cell = np.asarray(atoms, float)

    if cell.ndim == 2:
        c = cell.copy()
        cell = c.diagonal().copy()
        c.flat[::4] = 0.0
        if c.any():
            raise ValueError('Unit cell must be orthorhombic!')

    f = open(filename, 'w')
    np.array([3, 4], np.int32).tofile(f)

    dims = np.array(data.shape, np.int32)
    dims[::-1].tofile(f)

    for n, L in zip(dims[::-1], cell[::-1]):
        if n % 2 == 0:
            d = L / n
            np.array([0.0, L - d], np.float32).tofile(f)
        else:
            d = L / (n + 1)
            np.array([d, L - d], np.float32).tofile(f)

    if data.dtype == complex:
        data = np.abs(data)
    data.astype(np.float32).T.tofile(f)
    f.close()


def read_plt(fileobj):
    if isinstance(fileobj, basestring):
        fileobj = open(fileobj, 'rb')

    # dummy numbers
    np.fromfile(fileobj, dtype=np.int32, count=2)
    # read dimensions
    dims = np.fromfile(fileobj, dtype=np.int32, count=3)

    # read cell
    cell = np.zeros((3, 3), np.float32)
    for c in range(3):
        beg, Lmd = np.fromfile(fileobj, dtype=np.float32, count=2)
        n = dims[c]
        if n % 2 == 0:
            cell[2 - c, 2 - c] = Lmd / (1 - 1. / n)
        else:
            cell[2 - c, 2 - c] = Lmd / (1 - 1. / (n + 1))

    # read data
    data = np.fromfile(fileobj, dtype=np.float32)
    return data.reshape(dims).T, cell



from ase import Atom, Atoms
from ase.calculators.lj import LennardJones
from ase.constraints import FixBondLength

dimer = Atoms([Atom('X', (0, 0, 0)),
               Atom('X', (0, 0, 1))],
              calculator=LennardJones(),
              constraint=FixBondLength(0, 1))
print(dimer.get_forces())
print(dimer.positions)
dimer.positions[:] += 0.1
print(dimer.positions)
dimer.positions[:, 2] += 5.1
print(dimer.positions)
dimer.positions[:] = [(1, 2, 3), (4, 5, 6)]
print(dimer.positions)
dimer.set_positions([(1, 2, 3), (4, 5, 6.2)])
print(dimer.positions)



# creates: Al_phonon.png, Al_mode.gif
from ase.build import bulk
from ase.calculators.emt import EMT
from ase.phonons import Phonons

# Setup crystal and EMT calculator
atoms = bulk('Al', 'fcc', a=4.05)

# Phonon calculator
N = 7
ph = Phonons(atoms, EMT(), supercell=(N, N, N), delta=0.05)
ph.run()

# Read forces and assemble the dynamical matrix
ph.read(acoustic=True)
ph.clean()

path = atoms.cell.bandpath('GXULGK', npoints=100)
bs = ph.get_band_structure(path)

dos = ph.get_dos(kpts=(20, 20, 20)).sample_grid(npts=100, width=1e-3)

# Plot the band structure and DOS:
import matplotlib.pyplot as plt
fig = plt.figure(1, figsize=(7, 4))
ax = fig.add_axes([.12, .07, .67, .85])

emax = 0.035
bs.plot(ax=ax, emin=0.0, emax=emax)

dosax = fig.add_axes([.8, .07, .17, .85])
dosax.fill_between(dos.weights[0], dos.energy, y2=0, color='grey',
                   edgecolor='k', lw=1)

dosax.set_ylim(0, emax)
dosax.set_yticks([])
dosax.set_xticks([])
dosax.set_xlabel("DOS", fontsize=18)

fig.savefig('Al_phonon.png')

# --- literalinclude division line ---
from ase.io.trajectory import Trajectory
from ase.io import write

# Write modes for specific q-vector to trajectory files:
L = path.special_points['L']
ph.write_modes([l / 2 for l in L], branches=[2], repeat=(8, 8, 8), kT=3e-4,
               center=True)

# Generate gif animation:
with Trajectory('phonon.mode.2.traj', 'r') as traj:
    write('Al_mode.gif', traj, interval=50,
          rotation='-36x,26.5y,-25z')



from ase.build import molecule
from ase import optimize
from ase.vibrations.infrared import InfraRed

from gpaw.cluster import Cluster
from gpaw import GPAW, FermiDirac

h = 0.22

atoms = Cluster(molecule('H2'))
atoms.minimal_box(3.5, h=h)

# relax the molecule
calc = GPAW(h=h, occupations=FermiDirac(width=0.1))
atoms.set_calculator(calc)
dyn = optimize.FIRE(atoms)
dyn.run(fmax=0.05)
atoms.write('relaxed.traj')

# finite displacement for vibrations
ir = InfraRed(atoms)
ir.run()



from ase import Atoms
from ase.io import read, write
from ase.calculators.exciting import Exciting


a = Atoms('N3O',
          [(0, 0, 0), (1, 0, 0), (0, 0, 1), (0.5, 0.5, 0.5)],
          pbc=True)

write('input.xml', a)
b = read('input.xml')

print(a)
print(a.get_positions())
print(b)
print(b.get_positions())

calculator = Exciting(dir='excitingtestfiles',
                      kpts=(4, 4, 3),
                      # bin='/fshome/chm/git/exciting/bin/excitingser',
                      maxscl=3)



"""Try to import all ASE modules and check that ase.parallel.world has not
been used.  We want to delay use of world until after MPI4PY has been
imported.

We run the test in a subprocess so that we have a clean Python interpreter."""

import sys
from subprocess import run

# Should cover most of ASE:
modules = ['ase.optimize',
           'ase.db',
           'ase.gui']

imports = 'import ' + ', '.join(modules)

run([sys.executable,
     '-c',
     '{imports}; from ase.parallel import world; assert world.comm is None'
     .format(imports=imports)],
    check=True)



from ase.io.trajectory import Trajectory, PickleTrajectory
from ase.io.bundletrajectory import BundleTrajectory
from ase.io.netcdftrajectory import NetCDFTrajectory
from ase.io.formats import read, iread, write, string2index
__all__ = ['Trajectory', 'PickleTrajectory', 'BundleTrajectory',
           'NetCDFTrajectory', 'read', 'iread', 'write', 'string2index']



import numpy as np
from numpy.testing import assert_allclose
from ase.cell import Cell


TOL = 1E-10
rng = np.random.RandomState(0)

for i in range(20):
    cell0 = rng.uniform(-1, 1, (3, 3))
    for sign in [-1, 1]:
        cell = Cell(sign * cell0)
        rcell, Q = cell.standard_form()
        assert_allclose(rcell @ Q, cell, atol=TOL)
        assert_allclose(np.linalg.det(rcell), np.linalg.det(cell))
        assert_allclose(rcell.ravel()[[1, 2, 5]], 0, atol=TOL)



"""Inline viewer for jupyter notebook using X3D."""

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
from IPython.display import HTML

def view_x3d(atoms):
    """View atoms inline in a jupyter notbook. This command
    should only be used within a jupyter/ipython notebook.
    
    Args:
        atoms - ase.Atoms, atoms to be rendered"""
    
    output = StringIO()
    atoms.write(output, format='html')
    data = output.getvalue()
    output.close()
    return HTML(data)



""" test run for DFTB+ calculator
    the tolerance is extremely loose, beause different sk files
    give different results

"""
from ase.calculators.dftb import Dftb
from ase.optimize import QuasiNewton
from ase.build import molecule

test = molecule('H2O')
test.calc = Dftb(label='h2o')

dyn = QuasiNewton(test, trajectory='test.traj')
dyn.run(fmax=0.01)
final_energy = test.get_potential_energy()
assert abs(final_energy + 111.141945) < 1.0



import numpy as np
from ase.build import molecule
from ase.constraints import FixAtoms


def array_almost_equal(a1, a2, tol=np.finfo(type(1.0)).eps):
    """Replacement for old numpy.testing.utils.array_almost_equal."""
    return (np.abs(a1 - a2) < tol).all()


m = molecule('H2')
c = FixAtoms(indices=[atom.index for atom in m])
m.set_constraint(c)

pos1 = m.get_positions()
# shift z-coordinates by 1.
pos = m.get_positions()
pos[:, 2] += 1.

m.set_positions(pos)
# note that set_positions fails silently to set the new positions
# due to the presence of constraints!
assert array_almost_equal(pos1, m.get_positions())

m.positions = pos
# atoms.positions allows one to set the new positions
# even in the presence of constraints!
assert array_almost_equal(pos, m.get_positions())
